/***************************************************************************
                          carchive.cpp  -  description
                             -------------------
    begin                : Sat Dec 2 2000
    copyright            : (C) 2000 by Eric Coquelle
    email                : coquelle@caramail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "carchive.h"

#include <stdio.h>

#include <kiconloader.h>
#include <kfileitem.h>
#include <kdebug.h>

QListView* CArchive::list=NULL;
QComboBox* CArchive::combodirectories=NULL;
QString CArchive::archiveName="";
CLed* CArchive::led=NULL;
KStatusBar* CArchive::statusbar=NULL;
int CArchive::iconsize=16;
int CArchive::kindofdate=STANDARD_DATE;
QProgressBar* CArchive::progressbar=NULL;
bool CArchive::overwrite=true;
bool CArchive::recursivemode=true;
bool CArchive::displayicons=true;
int CArchive::compressrate=9;
bool CArchive::viewbydirectories=false;
QCString CArchive::archivePassword="";

CArchive::CArchive(){
	readArchiveWithStream=true;
	stopreadprocess=false;
  connect(&processread,SIGNAL(receivedStderr(KProcess*, char*, int)),this,SLOT(haveStdErr(KProcess*,char*,int)));
}

CArchive::~CArchive(){
}

/** display a message in the statusbar */
void CArchive::displayMessage( QString message ){
	if(statusbar==NULL)
		return;
  statusbar->clear();
  statusbar->changeItem(message, ID_STATUS_MSG);
}

/** Print in the statusbar the size of the loaded
	archive */
void CArchive::displayMessageArchiveSize(){
  QString message;

  message=QString("%1  -  %2 %3").arg(IDS_STATUS_DEFAULT).arg(archiveSize()).arg(i18n("bytes"));
  displayMessage(message);
}

/** return the size of current archive */
int CArchive::archiveSize(){
  FILE *F1;
  long archiveSize;

  archiveSize=0;
  if ((F1 = fopen(/*(const char*)*/archiveName, "rb")) == NULL)
    return 0;
  fseek(F1, 0, SEEK_END);
  archiveSize=ftell(F1);
  fclose(F1);
  return archiveSize;
}

/** Find which icon to insert in the listview, by
examinating the extension of files the archive contains */
void CArchive::definitIcone(QString name, QString /*permissions*/, CListViewItem* elementListe){
  if(!displayicons)
    return;

    KFileItem item(KFileItem::Unknown, KFileItem::Unknown, KURL("file:/"+name));
    elementListe->setPixmap(0,item.pixmap(iconsize,KIcon::DefaultState));
}

/** prepare the local variable, clear the listview
before reading any archive */
void CArchive::initializeReadingArchive(){
	stopreadprocess=false;
	counter=0;
  errors.clear();
  displayMessage(i18n("Running compressor..."));
  processread.clearArguments();
}

/** set the main widget so as to this class can
draw the archive content */
void CArchive::setWidgetListView( QListView *l ){
	list=l;
}

/** set the main widget so as to this class can
display the subfolders */
void CArchive::setWidgetComboDirectories( QComboBox *c ){
	combodirectories=c;
}

/** set an access to the led widget so as this
class can draw it */
void CArchive::setWidgetLed( CLed *l ){
	led=l;
}

/** set an access to the statusbar, so as this
class can draw in it */
void CArchive::setStatusBar( KStatusBar *b ){
	statusbar=b;
}

/** If @param read is true, read current archive
with a stream, ie by a quick way, but this
freezes karchiveur's interface */
void CArchive::setReadArchiveWithStream( bool read ){
	readArchiveWithStream=read;
}

void CArchive::displayArchiveContent() {
}

void CArchive::extractArchive(QString & , int , QString & ) {
}

void CArchive::removeFilesFromArchive(QStrList ) {
}

void CArchive::addFilesToArchive( QStrList , bool, int , const char* ) {
}

void CArchive::createArchive(QString , QStrList, const char* ) {
}

/** set the name of the working archive to
@param archName */
void CArchive::setArchiveName( QString archName ){
  archiveName=archName;
}

/** set the name of the password to
@param archPassword */
void CArchive::setPassword( QString archPassword){
  archivePassword=archPassword;
}

/** the the format of displayed date to d
 */
void CArchive::setKindOfDate( int d ){
	kindofdate=d;
}

/** if @param overwr is true, one will overwrite
files when extracting files */
void CArchive::setOverwriteFilesWhenExtracting( bool overwr ){
	overwrite=overwr;
}

/** A link to the progressbar */
void CArchive::setWidgetProgressBar( QProgressBar *progress ){
	progressbar=progress;
}

/** Kill the read process */
void CArchive::stopReading(){
	stopreadprocess=true;
	processread.kill();

}
/** Kill the extract process */
void CArchive::stopExtracting(){
	processextract.kill();
}

/** Set the size of the icons in the listview to @param size */
void CArchive::setIconSize( int size ){
	iconsize=size;
}

/** If true, when adding a directory, also add the subdirs */
void CArchive::setRecursiveMode (bool rec){
	recursivemode=rec;
}

/** if @param icons is false, don't display the
icons, to speed up archive reading */
void CArchive::setDisplayIcons( bool icons ){
	displayicons=icons;
}

/** If avaible for the compressor, set the compress
rate to @param rate. @param rate must be in [1,9] */
void CArchive::setCompressRate( int rate ){
	compressrate=rate;
}

void CArchive::haveStdErr(KProcess *, char *buffer, int length)
{
  QString sbuffer=buffer;
  sbuffer.truncate(length-1);
  errors.append(sbuffer);
}

QStringList CArchive::getErrors()
{
  return errors;
}

/* display archive content, using KIO methods */
/*KIO_KARCHIVE*/
void CArchive::displayArchiveContent(const KArchiveDirectory* tarfileContent, QString path)
{
  QStringList entries=tarfileContent->entries();
  KArchiveFile* entry;
  CListViewItem *elementListe;
  QString str;
  int i;

  for ( QStringList::Iterator it = entries.begin(); it != entries.end(); ++it ) {
    entry=(KArchiveFile*)tarfileContent->entry(*it);
    str=*it;
    if(entry->isDirectory())
    {
      elementListe=new CListViewItem(list,entry->name(),"0",entry->datetime().toString("hh:mm:ss"),entry->datetime().toString(), entry->user()+"("+entry->group()+")","",path);//+"/"+entry->name());
      definitIcone("..","",elementListe);
      if(viewbydirectories)
      {
        i=knowndirectories.findIndex((KArchiveDirectory*)tarfileContent);
        if(i==-1)
        {
          knowndirectories.append((KArchiveDirectory*)tarfileContent);
          combodirectories->insertItem(tarfileContent->name());
          combodirectories->setCurrentItem(combodirectories->count()-1);
        }
        else
          combodirectories->setCurrentItem(i);
      }
      else
        displayArchiveContent((const KArchiveDirectory*)tarfileContent->entry(*it),path+"/"+entry->name());
    }
    else
    {
      elementListe=new CListViewItem(list,entry->name(),QString::number(entry->size()),entry->datetime().toString("hh:mm:ss"),entry->datetime().toString(), entry->user()+"("+entry->group()+")","permissions",path+"/");
      definitIcone(entry->name(),"",elementListe);
    }
  }
}

/* display archive content, using KIO methods */
/*KIO_KARCHIVE*/
void CArchive::displayArchiveContent(QString foldername, QString folderpath)
{
  kdDebug()<<QString("displayArchiveContent of subdir %1, path=%2").arg(foldername).arg(folderpath)<<endl;
  KArchiveDirectory* currentdir=knowndirectories.last();
  KArchiveDirectory* requesteddir;
  QString requesteddirpath;
  KArchiveFile* entry=0L;
  bool found=false;
  QStringList entries=currentdir->entries();
  int i=0;

  //Check if foldername is a subfolder of currently displayed dir
  for ( QStringList::Iterator it = entries.begin(); it != entries.end(); ++it ) {
    entry=(KArchiveFile*)currentdir->entry(*it);
    if((entry->name()==foldername)&&(entry->isDirectory()))
    {
      found=true;
      break;
    }
  }
  if(found)
  {
    kdDebug()<<QString("It's a known subdir: %1").arg(folderpath+"/"+entry->name())<<endl;
    list->clear();
    displayArchiveContent((const KArchiveDirectory*)entry, folderpath+"/"+entry->name());
  }
  else
  {
    //Otherwise, check in the parent dirs
    for ( QValueList<KArchiveDirectory*>::Iterator it = knowndirectories.begin();  it!= knowndirectories.end(); it++ ) {
      requesteddir=*it;
      if(requesteddir->name()==foldername)
      {
        //Keep only the parents of current directory
        for(unsigned int j=i+1;j<knowndirectories.size();j++)
        {
          knowndirectories.remove(knowndirectories.last());
          combodirectories->removeItem(combodirectories->count()-1);
        }
        list->clear();
        displayArchiveContent(requesteddir, folderpath);
        break;
      }
      requesteddirpath+=requesteddir->name();//+"/";
      i++;
    }
  }
}

/** if true, KIO supported archives will be displayed dir. by dir. */
void CArchive::setDisplayArchiveByDirectories(bool b){
  viewbydirectories=b;
}

/** return a list of all files (with their archive's path) contained in the requested directory */
QStringList CArchive::getAllFiles(QString foldername, QString folderpath){
  KArchiveDirectory* currentdir=knowndirectories.last();
  //KArchiveDirectory* requesteddir;
  QString requesteddirpath;
  KArchiveFile* entry;
  QStringList entries=currentdir->entries();

  kdDebug()<<QString("Get all files from %1, in %2").arg(foldername).arg(folderpath)<<endl;
  //Check if foldername is a subfolder of currently displayed dir
  for ( QStringList::Iterator it = entries.begin(); it != entries.end(); ++it ) {
    entry=(KArchiveFile*)currentdir->entry(*it);
    if((entry->name()==foldername)&&(entry->isDirectory()))
      return getAllFiles((const KArchiveDirectory*)entry, folderpath+"/"+entry->name());
  }
    return NULL;
}

QStringList CArchive::getAllFiles(const KArchiveDirectory* tarfileContent, QString path)
{
  QStringList entries=tarfileContent->entries();
  KArchiveFile* entry;
  //CListViewItem *elementListe;
  QString str;
  QStringList fileswithpath;

  for ( QStringList::Iterator it = entries.begin(); it != entries.end(); ++it ) {
    entry=(KArchiveFile*)tarfileContent->entry(*it);
    str=*it;
    if(entry->isDirectory())
    {
      fileswithpath+=getAllFiles((const KArchiveDirectory*)tarfileContent->entry(*it),path+"/"+entry->name());
    }
    else
      fileswithpath.append(path+"/"+entry->name());
  }
  return fileswithpath;
}

/** Returns the password */
QCString CArchive::getPassword(){
 return archivePassword;
}

/* Set the files which can be extracted - return false if none of the files can be extracted*/
bool CArchive::checkFiles(QString extractpath, int extractall)
{
  QListViewItem *vi;
  int k,l;
  CCheckFiles check;
  QString filestoextract;
  QStringList listsubdir;
  
  check.setExtractPath(extractpath);
  l=0;
  vi=list->firstChild();

  for(k=0;k<list->childCount();k++)
  {
   if((strcmp(vi->text(0),"..")!=0)&&((vi->isSelected()&&(extractall==1))||(extractall!=1)))
  {
     if(viewbydirectories && supportDisplayByDirs())
       listsubdir=getAllFiles(vi->text(0),vi->text(6).startsWith("/") ? vi->text(6).right(vi->text(6).length()-1):vi->text(6));
     if((!viewbydirectories)||(!supportDisplayByDirs())||(viewbydirectories && supportDisplayByDirs() && listsubdir.isEmpty()))
     {
       filestoextract+=vi->text(6);
       filestoextract+=vi->text(0);
       filestoextract=filestoextract.right(filestoextract.length()-1);
       check.addFile(filestoextract);
     }
     else
     {
       for(QStringList::Iterator it = listsubdir.begin(); it != listsubdir.end(); ++it )
         kdDebug()<<QString("File in subdir:%1").arg(*it)<<endl;
       check.addFile(listsubdir);
     }

     filestoextract="";
     l++;
  	}
  	vi=vi->nextSibling();
  }
  progressbar->setTotalSteps(l);
  if(!check.canSafelyExtract())
    check.exec();
  listsubdir=check.getFiles();
  if(listsubdir.isEmpty())
    return false;
  processextract << listsubdir;
  return true;
}

#include "carchive.moc"
