/***************************************************************************
                          carchive.h  -  description
                             -------------------
    begin                : Sat Dec 2 2000
    copyright            : (C) 2000 by Eric Coquelle
    email                : coquelle@caramail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CARCHIVE_H
#define CARCHIVE_H

//KIO_KARCHIVE
#include <karchive.h>

#include <qstring.h>
#include <qstringlist.h>
#include <qlistview.h>
#include <qdatetime.h>
#include <qobject.h>
#include <qcombobox.h>
#include <qprogressbar.h>

#include <kstatusbar.h>
#include <kglobal.h>
#include <klocale.h>

#include "processus.h"
#include "cled.h"
#include "clistviewitem.h"
#include "ccheckfiles.h"
#include "resource.h"

#define STANDARD_DATE       0
#define LOCALISED_DATE      1
#define LOCALISED_FULL_DATE 2

/**This is the base class for all kind of archives
  *@author Eric Coquelle
  */

class CArchive : public QObject {
Q_OBJECT
public: 
	CArchive();
	~CArchive();
  /** set an access to the statusbar, so as this
		class can draw in it */
  void setStatusBar( KStatusBar *b );
  /** set an access to the led widget so as this
		class can draw it */
  void setWidgetLed( CLed *l );
  /** set the main widget so as to this class can
		draw the archive content */
  void setWidgetListView( QListView *l );

  /*virtual methods implemented in child classes*/
  virtual void displayArchiveContent();
  virtual void extractArchive(QString & extractpath, int extractall, QString & filetoextract);
  virtual void removeFilesFromArchive(QStrList filesToDelete);
  virtual void addFilesToArchive( QStrList filestoadd, bool removeoriginalfiles, int action, const char* relativepath=NULL);
	virtual void createArchive(QString nameofarchive, QStrList filestoadd, const char* relativepath=NULL);

  /** If @param read is true, read current archive
		with a stream, ie by a quick way, but this
		freezes karchiveur's interface */
  void setReadArchiveWithStream( bool read );
  /** set the name of the working archive to
		@param archName */
  void setArchiveName( QString archName );
  /** set the name of the password to
		@param archPassword */
  void setPassword( QString archPassword );
  /** the the format of displayed date to d
 */
  void setKindOfDate( int d );
  /** A link to the progressbar */
  void setWidgetProgressBar( QProgressBar *progress );
  /** if @param overwr is true, one will overwrite
		files when extracting files */
  void setOverwriteFilesWhenExtracting( bool overwr );
  /** Kill the extract process */
  void stopExtracting();
  /** Kill the read process */
  void stopReading();
  /** Set the size of the icons in the listview to @param size */
  void setIconSize( int size );
  /** If true, when adding a directory, also add the subdirs */
  void setRecursiveMode (bool rec);
  /** if @param icons is false, don't display the
icons, to speed up archive reading */
  void setDisplayIcons( bool icons );
  /** If avaible for the compressor, set the compress
rate to @param rate. @param rate must be in [1,9] */
  void setCompressRate( int rate );
  /*get the StdErr of compressors*/
  QStringList getErrors();
  /*Display archive content, using KIO methods */
  //KIO_KARCHIVE
  void displayArchiveContent(QString , QString );
  /** if true, KIO supported archives will be displayed dir. by dir. */
  void setDisplayArchiveByDirectories(bool);
  /** return a list of all files (with their archive's path) contained in the requested directory */
  QStringList getAllFiles(QString, QString);
  QStringList getAllFiles(const KArchiveDirectory*, QString);
  /** set the main widget so as to this class can
  display the subfolders */
  void setWidgetComboDirectories( QComboBox *c );
  /*Returns false as all archives cannot be listed by KIO*/
  virtual bool supportDisplayByDirs(){return false;};
  /*Returns true if archive type supports passwords*/
  virtual bool supportPassword(){return false;};
  /** Returns the password */
  QCString getPassword();
  /* Set the files which can be extracted - return false if none of the files can be extracted*/
  bool checkFiles(QString, int);
  
protected:
	/* The cprocessus object, it will be used to list archive content...*/
	CProcessus processread;
	/* another one to extract files from archive */
	CProcessus processextract;
	/* The counter for the progressbar*/
	int counter;
	/* The name of the archive we are working on*/
	static QString archiveName;
	/* If true, read the archive using the CProcessus class.
		 Quicker, but freeze the interface */
	bool readArchiveWithStream;
	/* A pointer to the statusbar's led */
	static CLed *led;
	/* A pointer to the statusbar */
	static KStatusBar* statusbar;
	/* A pointer to the listview (where archive content is displayed */
	static QListView *list;
	/* if readArchiveWithStream is false, we store in a buffer the output
	 * of the running command */
	char m_buffer[1024];
	/* The size of icons in the listview*/
	static int iconsize;
	/*Has user canceled current process ?*/
	bool stopreadprocess;
	/*option for displaying the date*/
	static int kindofdate;
	/*A pointer to the progressbar*/
	static QProgressBar *progressbar;
	/*If true, overwrite files when extracting*/
	static bool overwrite;
	/*If true, add to the archive subdirectories*/
	static bool recursivemode;
	/*Display icons for archive content ?*/
	static bool displayicons;
	/*The compress rate, if avaible: 1=min, 9=max*/
	static int compressrate;
  /*The error messages*/
  QStringList errors;
  /* View mode: directory bu directory, or all files at once */
  static bool viewbydirectories;
  static QComboBox* combodirectories;
  /* Current directory*/
  // KIO_ARCHIVE
  QValueList<KArchiveDirectory*> knowndirectories;
	
protected:
  /** Password for archive */
  static QCString archivePassword;
  /** return the size of current archive */
  int archiveSize();
  /** Print in the statusbar the size of the loaded
		archive */
  void displayMessageArchiveSize();
  /** display a message in the statusbar */
  void displayMessage( QString message );
	/** Find which icon to insert in the listview, by
		examinating the extension of files the archive contains */
	void definitIcone(QString name, QString permissions, CListViewItem* elementListe);
  /** prepare the local variable, clear the listview
before reading any archive */
  void initializeReadingArchive();
  /*Display archive content, using KIO methods */
  //KIO_KARCHIVE
  void displayArchiveContent(const KArchiveDirectory*, QString);
  
protected slots:
  void haveStdErr(KProcess *, char *buffer, int length);
    
public:
signals:
	void archiveReadEnded(void);
};

#endif
