/***************************************************************************
                          carj.cpp  -  description
                             -------------------
    begin                : Sat Dec 2 2000
    copyright            : (C) 2000 by Eric Coquelle
    email                : coquelle@caramail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "carj.h"
#include <kdebug.h>
#include <qdir.h>

CArj::CArj(){
	CArchive();

	connect(&processread,SIGNAL(receivedStdout(KProcess*, char*, int)),this,SLOT(haveSdtOut(KProcess*,char*,int)));
	connect(&processread,SIGNAL(processExited (KProcess*)),this,SLOT(endProcess(KProcess*)));
	connect(&processextract,SIGNAL(receivedStdout(KProcess*, char*, int)),this,SLOT(haveSdtOutExtract(KProcess*,char*,int)));
	connect(&processextract,SIGNAL(processExited (KProcess*)),this,SLOT(endProcess(KProcess*)));

	list->setColumnText(4, i18n("Owner"));
	list->setColumnText(5, i18n("Permissions"));
}

CArj::~CArj(){
}

/** this method will launch the right compressor
	to list the file contents
	It will then launch the right method to display
	the content in the listview */
void CArj::displayArchiveContent(){
	FILE *flot;

	initializeReadingArchive();

  processread << "unarj";
	processread << "l" << archiveName;

  if(readArchiveWithStream)
  {
		processread.demarrer(&flot,KProcess::AllOutput);
		displayArjArchiveContent(flot);
		displayMessageArchiveSize();
		led->setColor(KLed::green);
	}
	else
	{
		m_buffer[0]='\0';
		headerremoved=false;
		finished=false;
		processread.start(KProcess::NotifyOnExit, KProcess::AllOutput);
	}
}

/** display in a listview the content of the current
arj archive. This method examines the stdout of
unarj to sort useful infos for the listview */
void CArj::displayArjArchiveContent(FILE* flot){
	CListViewItem* elementListe;
	char tampon[5000];
	char permission[30];
	char proprietaire[256];
	char crc[20];
	char taille[30];
	char date[12];
	char heure[7];
	char snom[5000];
	QString path;
	QString nom;
	QString message;
	int i;

	for(i=0;i<6;i++)
	{
		fgets( tampon, 5000, flot );
		sscanf(tampon,"%[^\n]",snom);
	}

	fgets( tampon, 5000, flot );

	while( !feof(flot) && (strstr( tampon, "------------" )==NULL))
	{
		sscanf(tampon, "%[a-zA-Z.0-9~] %[0-9] %[0-9] %[0-9.] %[0-9-] %[0-9:] %[^\n]", snom, taille, permission, crc, date, heure, proprietaire );

		nom=snom;
		nom=" "+nom;
		i=nom.findRev('/');
		if(i!=-1)
		{
			path=nom.left(i+1);
		 	nom=nom.remove(0,i+1);
		if(nom=="")
			nom="..";
		}
		else
			path="";

		elementListe=new CListViewItem(list,nom,taille,heure,date,"","",path);
		definitIcone(nom, permission, elementListe);
		elementListe->widthChanged();
		fgets( tampon, 5000, flot );
	}
}

/*Overloaded method*/
void CArj::displayArjArchiveContent( const char* ligne){
	CListViewItem* elementListe;
	char permission[30];
	char proprietaire[256];
	char crc[20];
	char taille[30];
	char date[12];
	char heure[7];
	char snom[5000];
	QString path;
	QString nom;
	QString message;
	int i;

		sscanf(ligne, "%[a-zA-Z.0-9~] %[0-9] %[0-9] %[0-9.] %8[0-9] %[0-9:] %[^\n]", snom, taille, permission, crc, date, heure, proprietaire );

		nom=snom;
		nom=" "+nom;
		i=nom.findRev('/');
		if(i!=-1)
		{
			path=nom.left(i+1);
		 	nom=nom.remove(0,i+1);
		if(nom=="")
			nom="..";
		}
		else
			path="";

		elementListe=new CListViewItem(list,nom,taille,heure,date,"","",path);
		definitIcone(nom, permission, elementListe);
		elementListe->widthChanged();
}

/** we recive some informations through the standard
output of the process */
void CArj::haveSdtOut(KProcess *, char *buffer, int length){
	//Has user canceled current action ?
	if(stopreadprocess)
		return;

	// This section is here only for testing. It has been taken from ark:
	//1997-1999: Rob Palmbos palm9744@kettering.edu
	//2000: Corel Corporation (author: Emily Ezust, emilye@corel.com)
	//and adapted to karchiveur
	int passage;
  char c = buffer[length];
  buffer[length] = '\0';

//  m_settings->appendShellOutputData( buffer );

  char line[1024] = "";
  char *tmpl = line;
  char *tmpb;

  displayMessage(i18n("Running compressor..."));


  //We copy m_buffer to tmpl
  for( tmpb = m_buffer; *tmpb != '\0'; tmpl++, tmpb++ )
    *tmpl = *tmpb;

  //We copy the fisrt string of buffer (till \n) to tmpl
  for( tmpb = buffer; *tmpb != '\n'; tmpl++, tmpb++ )
    *tmpl = *tmpb;

  tmpb++;
  *tmpl = '\0';

  if( *tmpb == '\0' )
    m_buffer[0]='\0';

/*	if((strncmp(line,"UNARJ",5)!=0)&&(line[0]!='\n'))
		displayArjArchiveContent( line );*/

  if( !strstr( line, "----" ) )
  {
		if( headerremoved && !finished )
	  	displayArjArchiveContent( line );
	}
  else if(!headerremoved)
    headerremoved = true;
  else
    finished = true;

  bool stop = (*tmpb == '\0');
  passage=0;

  while( !stop && !finished)
  {
      tmpl = line; *tmpl = '\0';

      for(; (*tmpb!='\n') && (*tmpb!='\0'); tmpl++, tmpb++)
				*tmpl = *tmpb;

      if( *tmpb == '\n' )
			{
			  *tmpl = '\n';
			  tmpl++;
			  *tmpl = '\0';
			  tmpb++;

  if( !strstr( line, "----" ) )
  {
		if( headerremoved && !finished )
	  	displayArjArchiveContent( line );
	}
  else if(!headerremoved)
    headerremoved = true;
  else
    finished = true;

	  	/*if((passage!=2)&&(strncmp(line,"-------- ",9)!=0)&&(strncmp(line," Length   Method",16)!=0))
	  		displayArjArchiveContent( line );
	  	else
	  		passage++;*/
			}
      else if (*tmpb == '\0' )
			{
	  		*tmpl = '\0';
			  strcpy( m_buffer, line );
	  		stop = true;
			}
    }
  buffer[length] = c;
}

/** we recive some informations through the standard
output of the process */
void CArj::haveSdtOutExtract(KProcess *, char *buffer, int length){
	QString inter;
	int i=0;

	inter=buffer;
	inter=inter.left(length);

	while((i=inter.find('\n',i+1))!=-1){
		progressbar->setProgress(progressbar->progress()+1);
	}
}

/** The current process ended */
void CArj::endProcess(KProcess*){
  kdDebug()<<("Process ENDED\n");
	led->setColor(QColor("green"));
	displayMessageArchiveSize();
	progressbar->reset();

	emit(archiveReadEnded());
}

/** Upon the kind of archive, choose the right
	*uncompressor and extract all or some files
	*@toutextraire = 9: extract to karchiveur's temp directory (for viewing)
	*@toutextraire = 1: extract all selected files
	*/
void CArj::extractArchive(QString & extractpath, int , QString & ){
 	QString directoryOption;
 	QString nomextrait;
 	QString fichiersaextraire;

	directoryOption="";
	fichiersaextraire="";
	counter=0;
	progressbar->reset();
	progressbar->setTotalSteps(list->childCount());

	QDir::setCurrent(extractpath);
	processextract.clearArguments();
	processextract << "unarj";
	processextract << "e";
	if(!overwrite)
		processextract << "-k";

	processextract << archiveName;

	//We can't extract some selected files with unarj demo
	//We extract through the Stdout to have a progress indicator
	if(processextract.start(KProcess::NotifyOnExit,KProcess::Stdout)==FALSE)
	  kdDebug()<<("\n*PB PROCESS*\n");

 	counter=0;
}
#include "carj.moc"
