/***************************************************************************
                          ccheckfiles.cpp  -  description
                             -------------------
    begin                : mar jan 20 2004
    copyright            : (C) 2004 by Eric Coquelle
    email                : coquelle@caramail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kiconloader.h>
#include <klocale.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qfileinfo.h>
#include <qpixmap.h>
#include <qdir.h>

#include "ccheckfiles.h"

CCheckFiles::CCheckFiles(){
  KIconLoader icon;
  setIcon(icon.loadIcon("karchiver",KIcon::Small));

  QGridLayout *grid = new QGridLayout(this,5,3,15,7);

  QLabel* label_1;
  label_1 = new QLabel( this, "Label_1" );
  label_1->setText( i18n("The following files from your archive will overwrite those\n of your disk. Unselect from the list the files you don't want to extract.") );
  grid->addMultiCellWidget(label_1,0,0,0,2);

  files = new KListBox( this, "ListBox_files" );
  files->setSelectionMode(QListBox::Multi);
  grid->addMultiCellWidget(files,1,3,0,2);

  ok=new QPushButton(i18n("Proceed"), this, "button_ok");
  grid->addWidget(ok,4,1);
  connect(ok, SIGNAL(clicked()), SLOT(accept()));

  resize( 490,310 );
}

CCheckFiles::~CCheckFiles(){
}

/** fills the listbox */
void CCheckFiles::addFile(QStringList list){
  for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )  
    checkSafelyExtract(*it);
}

/** fills the listbox */
void CCheckFiles::addFile(QString str){
  checkSafelyExtract(str);
}

/** returns the files that can be extracted */
QStringList CCheckFiles::getFiles(){
  QStringList list;

  for (QListBoxItem *item=files->firstItem(); item; item = item->next())
  {
    if(files->isSelected(item))
      list.append(item->text());
  }

  return list+filescanbeextracted;
}

/** Set the path where the files will be extracted */
void CCheckFiles::setExtractPath(QString str){
  QFileInfo fi(str);
  extractpath=str+"/";
  canwritebasepath=checkExtractPath(extractpath);
}

/** return true if the present file can be safely extracted, e.g. it will neither overwrite a file nor goes to an unwritable directory */
bool CCheckFiles::checkSafelyExtract(QString name){
  if(name.startsWith("/"))
    name=name.remove(0,1);
  QString fullfilename=extractpath+name;
  QFileInfo info(fullfilename);

  if(!canwritebasepath)
  {
      KIconLoader icon;
      filescannotbeextracted.append(name);
      files->insertItem(icon.loadIcon("cancel",KIcon::Small),name);
      return false;
  }  
  else if(info.exists())
    if(info.isWritable())
    {
      filesaskbeforeextract.append(name);
      files->insertItem(name);
      files->setSelected(files->count()-1,true);
    }
    else
    {
      KIconLoader icon;
      filescannotbeextracted.append(name);
      files->insertItem(icon.loadIcon("cancel",KIcon::Small),name);
      return false;
    }
  else
    filescanbeextracted.append(name);
  return true;      
}

/** return true if all the files can be extracted without any user action */
bool CCheckFiles::canSafelyExtract(){
  if(filesaskbeforeextract.isEmpty()&&filescannotbeextracted.isEmpty())
    return true;
  return false;
}

/** Check if the extract path exists. Otherwise, create it if possible */
bool CCheckFiles::checkExtractPath(QString d){
  QDir dir;
  QFileInfo f;
  dir.setPath(d);

  if(dir.exists())
  {
    f.setFile(d);
    if(f.isWritable())
      return true;
    else
      return false;
  }
  else
  {
    int pos=d.findRev("/");
    if(pos==-1)
      return false;

    if(checkExtractPath(d.left(pos)))
    {
      dir.setPath(d);
      dir.mkdir(d);
      return true;
    }
  }
  return false;  
}

#include "ccheckfiles.moc"
