/***************************************************************************
                          cfiledialog.cpp  -  description
                             -------------------
    begin                : mer sep 24 2003
    copyright            : (C) 2003 by Eric Coquelle
    email                : coquelle@caramail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kurlcombobox.h>
#include <kfilefiltercombo.h>
#include <klocale.h>

#include "cfileinfo.h"
#include "cfiledialog.h"

CFileDialog::CFileDialog(const QString& startDir, const QString& filter, QWidget *parent, const char *name, bool modal, QWidget* widget) : KFileDialog(startDir, filter, parent, name, modal, widget)
{  
  init();
}

CFileDialog::CFileDialog(const QString& startDir, const QString& filter, QWidget *parent, const char *name, bool modal) : KFileDialog(startDir, filter, parent, name, modal)
{
  init();
}

void CFileDialog::init()
{
  QString filter;
  filter="*.tar.gz *.tgz *.Z|";
  filter+=i18n("targz archives");
  filter+="\n*.tar.bz2|";
  filter+=i18n("tarbz2 archives");
  filter+="\n*.lha *.lzh|";
  filter+=i18n("lha archives");
  filter+="\n*.arj|";
  filter+=i18n("arj archives");
  filter+="\n*.rar|";
  filter+=i18n("rar archives");
  filter+="\n*.gz|";
  filter+=i18n("pure gzip archives");
  filter+="\n*.bz2|";
  filter+=i18n("pure bzip2 archives");
  filter+="\n*.zip|";
  filter+=i18n("zip archives");
  filter+="\n*.tar|";
  filter+=i18n("pure tar archives");

  setFilter(filter);
  
  connect(this, SIGNAL(filterChanged ( const QString& )),  SLOT(slotDialogNewArchiveFilterChanged(const QString& )));
  connect(locationEdit, SIGNAL(textChanged ( const QString& )),  SLOT(slotDialogNewArchiveFileSelected(const QString& )));
}

CFileDialog::~CFileDialog(){
}

void CFileDialog::slotDialogNewArchiveFilterChanged(const QString& filter)
{
  QString currentarchivename;
  CFileInfo f(locationEdit->currentText());
  QString extension;
  
  if(f.baseName().isEmpty())
    return;
    
  if(filter=="*.tar.gz *.tgz *.Z")
    extension=".tar.gz";
  else if(filter=="*.tar.bz2")
    extension=".tar.bz2";
  else if(filter=="*.lha *.lzh")
    extension=".lha";
  else if(filter=="*.arj")
    extension=".arj";
  else if(filter=="*.rar")
    extension=".rar";
  else if(filter=="*.gz")
    extension=".gz";
  else if(filter=="*.bz2")
    extension=".bz2";
  else if(filter=="*.zip")
    extension=".zip";
  else if(filter=="*.tar")
    extension=".tar";

  currentarchivename=f.baseName()+extension;
  locationEdit->changeItem(currentarchivename,locationEdit->currentItem());
}

void CFileDialog::slotDialogNewArchiveFileSelected(const QString& file)
{
  QString currentarchivename;
  QFileInfo f(file);
  QString extension=f.extension().lower();

  if(f.extension().isEmpty())
    return;

  if((extension=="tar.gz")||(extension=="tgz"))
    filterWidget->setCurrentItem(0);
  else if(extension=="tar.bz2")
    filterWidget->setCurrentItem(1);
  else if((extension=="lha")||(extension=="lhz"))
    filterWidget->setCurrentItem(2);
  else if(extension=="arj")
    filterWidget->setCurrentItem(3);
  else if(extension=="rar")
    filterWidget->setCurrentItem(4);
  else if(extension=="gz")
    filterWidget->setCurrentItem(5);
  else if(extension=="bz2")
    filterWidget->setCurrentItem(6);
  else if(extension=="zip")
    filterWidget->setCurrentItem(7);
  else if(extension=="tar")
    filterWidget->setCurrentItem(8);
}

/* If selected file doesn't have an extension, add one according to the selected filter*/
QString CFileDialog::fixFileExtension(QString& file)
{
  QString filter;
  CFileInfo f(file);
  
  if(f.extension().isEmpty()&&(!file.isEmpty()))
  {
    filter=currentFilter();
    if(filter=="*.tar.gz *.tgz *.Z")
      filter=".tar.gz";
    else if(filter=="*.tar.bz2")
      filter=".tar.bz2";
    else if(filter=="*.lha *.lzh")
      filter=".lha";
    else if(filter=="*.arj")
      filter=".arj";
    else if(filter=="*.rar")
      filter=".rar";
    else if(filter=="*.gz")
      filter=".gz";
    else if(filter=="*.bz2")
      filter=".bz2";
    else if(filter=="*.zip")
      filter=".zip";
    else if(filter=="*.tar")
      filter=".tar";
    file+=filter;
  }
  return file;
}  
#include "cfiledialog.moc"
