/***************************************************************************
                          ctarbz2.cpp  -  description
                             -------------------
    begin                : Sat Dec 2 2000
    copyright            : (C) 2000 by Eric Coquelle
    email                : coquelle@caramail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ctarbz2.h"
#include <kdebug.h>
CTarBz2::CTarBz2(){
	CTar();
}

CTarBz2::~CTarBz2(){
}

/** this method will launch the right compressor
to list the file contents
It will then launch the right method to display
the content in the listview */
void CTarBz2::displayArchiveContent(){
//KIO_KARCHIVE
  if(viewbydirectories)
    CTar::displayArchiveContent();
  else
  {
    FILE *flot;

  	initializeReadingArchive();

  	processread << "tar";
  	processread << "--use-compress-program=bzip2" << "-vtf" << archiveName;

  	if(readArchiveWithStream)
  	{
  			processread.demarrer(&flot,KProcess::AllOutput);
  			displayTarArchiveContent(flot);
  			displayMessageArchiveSize();
  			led->setColor(KLed::green);
  	}
  	else
  	{
  		m_buffer[0]='\0';
  		processread.start(KProcess::NotifyOnExit, KProcess::AllOutput);
  	}
  }
}

/** Upon the kind of archive, choose the right
	*uncompressor and extract all or some files
	*@toutextraire = 9: extract to karchiveur's temp directory (for viewing)
	*@toutextraire = 1: extract all selected files
	*/
void CTarBz2::extractArchive(QString &extractpath, int extractall, QString & filetoextract){
 	QString optionRepertoire;
 	QString nomextrait;
 	QString fichiersaextraire;
  QStringList listsubdir;

 	optionRepertoire="";
 	fichiersaextraire="";
 	counter=0;
 	progressbar->reset();
  progressbar->setTotalSteps(list->childCount());

 	processextract.clearArguments();
 	processextract << "tar";

	optionRepertoire="-C";
	optionRepertoire+=extractpath;
	processextract << "--use-compress-program=bzip2";
	processextract << "-vxf";
	if(!overwrite)
		processextract << "-k";

 	processextract << archiveName;
  processextract << optionRepertoire;

 	//We will extract all selected files
  if((extractall!=EXTRACTANDOVERWRITE)&&(extractall!=EXTRACTONE_ANDBLOCK)&&(!checkFiles(extractpath, extractall)))
  {
    endProcess(NULL);
    return;
  }

  //We want to view (and so extract) only one file. So we just add this file to the tar or unzip
 	//command. For gzip and bzip2 files, in any case, we extract one and only one file, so I put
 	//it apart
 	else if(extractall==EXTRACTONE_ANDBLOCK)
 	{
 		processextract << filetoextract;
 		if(processextract.start(KProcess::Block)==FALSE)
		  kdDebug()<<("\n*PB PROCESS*\n");
 	}

 	//We extract through the Stdout to have a progress indicator
 	if(extractall!=EXTRACTONE_ANDBLOCK)
 	{
 		if(processextract.start(KProcess::NotifyOnExit,KProcess::Stdout)==FALSE)
		  kdDebug()<<("\n*PB PROCESS*\n");
 	}
 	counter=0;
}

/** delete @param filestodelete from current archive */
void CTarBz2::removeFilesFromArchive( QStrList filestodelete ){
	processextract.clearArguments();
	processextract << "bzip2" << "-df" << archiveName;
	processextract.start(KProcess::Block);

	//The name of archive arch.tar.bz2 is now arch.tar
	archiveName=archiveName.left(archiveName.length()-4);
	CTar::removeFilesFromArchive(filestodelete);

	processread.clearArguments();
	processread << "bzip2" <<archiveName;
	processread.start(KProcess::Block);
	archiveName+=".bz2";
}

/** Add some files to the archive
@param filestoadd : list of files to add
@param removeoriginalfiles : remove or not those files from disk
@param action : 0=mode append and replace files, 1=mode update files
@param relativepath : if !NULL, include only filenames, without their base path */
void CTarBz2::addFilesToArchive( QStrList filestoadd, bool removeoriginalfiles, int action, const char* relativepath){
  kdDebug()<<("\nDecompression BZIP2\n");
 	processread.clearArguments();
 	processread << "bzip2" << "-df" << archiveName;
 	processread.start(KProcess::Block);

	//The name of archive arch.tar.b2z is now arch.tar
	archiveName=archiveName.left(archiveName.length()-4);

	kdDebug()<<("\nDecompression BZIP2 OK\n");
 	//We add files to the pure tar archive
 	CTar::addFilesToArchive(filestoadd, removeoriginalfiles, action, relativepath);
	kdDebug()<<("\nADD TAR OK\n");
	//Then we compress with gzip
 	processread.clearArguments();
 	processread << "bzip2" << archiveName;
 	processread.start(KProcess::Block);

	archiveName+=".bz2";
	kdDebug()<<("\nCompression BZIP2 OK\n");
}

/** Create a targ.bz2archive
@param  nameofarchive: the name of the targz archive
@param param: list of files to add
@param relativepath: include only filenames, without their path */
void CTarBz2::createArchive(QString nameofarchive, QStrList filestoadd, const char* relativepath) {
	archiveName=nameofarchive;
	//The name of archive arch.tar.b2z is now arch.tar
	archiveName=archiveName.left(archiveName.length()-4);

 	//We add files to the pure tar archive
 	CTar::addFilesToArchive(filestoadd, false, 0, relativepath);
	kdDebug()<<("\nADD TAR OK\n");
 	//Then we compress with gzip
 	processread.clearArguments();
 	processread << "bzip2" << archiveName;
 	processread.start(KProcess::Block);

	archiveName+=".bz2";
	kdDebug()<<("\nCompression BZIP2 OK\n");
}
#include "ctarbz2.moc"
