/***************************************************************************
                          karchiveur.cpp  -  description
                             -------------------
    begin                : mer jun 14 00:26:31 CEST 2000
    copyright            : (C) 2000 by Eric Coquelle
    email                : coquelle@caramail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 
// system includes
#include <errno.h>
#include <math.h>

// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>

// include files for KDE
#include <kdeversion.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kfileitem.h>
#include <kopenwith.h>
#include <kstdaction.h>
#include <kurldrag.h>
#include <kedittoolbar.h>
#include <kdebug.h>
#include <klineeditdlg.h>
#include <kpopupmenu.h>
#include <karchive.h>
#include <ktar.h>
#include <kzip.h>
#include <kpassdlg.h>

// application specific includes
#include "cfileinfo.h"
#include "cfiledialog.h"
#include "karchiveur.h"
#include "karchiveurview.h"
#include "cgpg.h"
#include "cmail.h"
#include "cupdate.h"
#include "resource.h"


KarchiveurApp::KarchiveurApp()
{
	archiveobj = new CArchive();
  cbdirectories=NULL;
	config=kapp->config();

	///////////////////////////////////////////////////////////////////
	// call inits to invoke all other construction parts
	initStatusBar();
	initActions();
	initView();
	enableMenus(false);

	archivechoice = new CArchiveChoice(this, "Identify_and_Open_Archive_Class");
	archivechoice->hide();
	extraction = new CExtraction(0, "Extraction_Dialog");
	extraction->hide();

	//Launch the archive browser
	navigateur = new CArchiveBrowser(this,"File_Browser_Dialog");
	connect( navigateur, SIGNAL(windowwillbeclosed()), this, SLOT(slotHideButineur()));
	connect(navigateur->QListBox_1, SIGNAL(selected(const QString&)), this, SLOT(slotOpenFileFromArchiveBrowser(const QString&)));

  //Set toolbar directory browser
  cbdirectories=new KComboBox(toolBar("mainToolBar"), "toolbar_combo");
  connect(cbdirectories, SIGNAL(activated(int)), this, SLOT(slotComboDirectoriesActivated(int)));

	// reads in all config-options
	readOptions();

	//Additionnal init to the statusbar
//	progress=new KProgress(statusBar(),"barre progression");
	progress = new QProgressBar(statusBar(),"Progress_Bar");
	progress->setFixedHeight ( statusBar()->sizeHint().height() -4 );
	statusBar()->addWidget(progress,0,true);
//	progress->setGeometry(150,3,200,statusBar()->height()-3); // Not needed anymore (mETz)

	led=new CLed(statusBar(),"statusbar_led");
	led->setLook(KLed::Sunken);
	led->setFixedHeight ( statusBar()->sizeHint().height()-4 );
	led->setFixedWidth ( statusBar()->sizeHint().height()-4 );
	statusBar()->addWidget(led,0,true);
//	led->setGeometry(statusBar()->width()-20,9,19,statusBar()->height()-3); // Not needed anymore (mETz)

	//The tip of the day
	tipofday = new KTipofDay(this,"Tip_Of_Day_Dialog");
	if(displaytipofday)
		tipofday->show();

	operation=NOARCHIVE; //No file loaded, nothing to do !
	optionExtract=1; //Default extraction: in home dir
	optionOpening=1; //Default opening: in home dir
	recursive=true; //when creating an archive, aslo compress subfolders

	//Initialise for drops
	//The main drop zone is in KarchiveurView (via QListView* liste)
	setAcceptDrops(true);
	connect(view,SIGNAL(faireGlisserDeposer()), this, SLOT(slotDoDragAndDrop()));
	connect(view,SIGNAL(rightButtonPressed ( QListViewItem *, const QPoint &, int )), this, SLOT(slotShowContextualMenu ( QListViewItem *, const QPoint &, int )));
	connect(view,SIGNAL(clicked ( QListViewItem * )), this, SLOT(slotMouseClick ( QListViewItem * )));

	//Initialise the process object:
	process=new CProcessus();
	process->clearArguments();
	connect(process,SIGNAL(receivedStdout(KProcess*, char*, int)),this,SLOT(recueSdtOut(KProcess*,char*,int)));
	connect(process,SIGNAL(processExited (KProcess*)),this,SLOT(finProcessus(KProcess*)));

	//Initialise the find in this archive tool
	find = new CFind(this,"Search_in_this_Archive_Class");
	connect( find, SIGNAL(findNext(QString)), this, SLOT(slotFindInThisArchive(QString)));
	connect( find, SIGNAL(findClose()), this, SLOT(slotCloseFind()));

//	navigateur->exec();
//	navigateur->show();

	//Initialise the find archives on disk tool
	searchBox=new CSearch(0,"Search_for_Archives_Dialog");
	connect(searchBox->foundarchives, SIGNAL(selected(const QString&)), this, SLOT(slotOpenFileFromArchiveFinder(const QString&)));

	 QString directory;
   QString error;
	 directory.sprintf("karchivertmp.%d/", getpid());
   error=strerror(errno);
	 reptmp = locateLocal( "tmp", directory );

	 if (reptmp.isNull())
	   kdWarning() << i18n("Failed to create temporary "
		    "attachment directory %2: %1")
		    .arg(error)
		    .arg(directory) << endl;


	 directory.sprintf("karchivertmp.%d/extract/", getpid());
	 reptmp = locateLocal( "tmp", directory );

	 if (reptmp.isNull())
	   kdWarning() <<i18n("Failed to create temporary "
		    "attachment directory %2: %1")
		    .arg(error)
		    .arg(directory) << endl;

	 directory.sprintf("karchivertmp.%d/undo/", getpid());
	 reptmpundo = locateLocal( "tmp", directory );

	 if (reptmpundo.isNull())
	   kdWarning() <<i18n("Failed to create temporary "
		    "attachment directory %2: %1")
		    .arg(error)
		    .arg(directory) << endl;

	slotViewButineur();
	slotViewButineur();

	connect(archiveobj, SIGNAL(archiveReadEnded()), this, SLOT(slotEndProcess()));
	archiveobj->setWidgetListView ( view );
  archiveobj->setWidgetComboDirectories(cbdirectories);
	archiveobj->setStatusBar(statusBar());
	archiveobj->setWidgetLed(led);
	archiveobj->setWidgetProgressBar(progress);
	archiveobj->setArchiveName("");
	archiveobj->setOverwriteFilesWhenExtracting(overwritegzip);
	archiveobj->setIconSize(16);
	archiveobj->setReadArchiveWithStream(readarchivewithstream);
	delete archiveobj;
	archiveobj=NULL;
}

KarchiveurApp::~KarchiveurApp()
{
  QString directory;
  directory.sprintf("karchivertmp.%d/", getpid());
  reptmp = locateLocal( "tmp", directory );

  process->clearArguments();
  *process << "rm" << "-Rf" << reptmp;
  process->start(KProcess::Block);

  if ( archiveobj != NULL )
 	delete archiveobj;

// Why clear the clipboard on exit? (mETz)
//	kapp->clipboard()->clear();
}

// Not needed anymore (mETz)
/*
void KarchiveurApp::resizeEvent( QResizeEvent * )
{
	int largeur;
	int hauteur;

	largeur=width()-1;
	hauteur=height();

#warning FIXME ListView should resize
	if(viewStatusbar->isChecked())
		hauteur-=75;
	else
		hauteur-=55;

	view->setGeometry(0,0,largeur,hauteur);
	led->setGeometry(statusBar()->width()-20,4,statusBar()->height()-5,statusBar()->height()-5);
	progress->setGeometry(width()/3,3,width()/3,statusBar()->height()-3);
}
*/
void KarchiveurApp::initStatusBar()
{
	///////////////////////////////////////////////////////////////////
	// STATUSBAR
	// TODO: add your own items you need for displaying current application status.
	statusBar()->insertItem(i18n(IDS_STATUS_DEFAULT), ID_STATUS_MSG);
}

void KarchiveurApp::initView()
{
	////////////////////////////////////////////////////////////////////
	// create the main widget here that is managed by KTMainWindow's view-region and
	// connect the widget to your document to display document contents.
	view = new KarchiveurView(this, "listefichiersarchive"); /* (this); */
	setCentralWidget(view);
}

void KarchiveurApp::saveOptions()
{
	config->setGroup("General Options");
	config->writeEntry("Geometry", size());
	config->writeEntry("Show Toolbar", viewToolbar->isChecked());
	config->writeEntry("Show Statusbar",viewStatusbar->isChecked());
	config->writeEntry("Show Navigateur",viewArchiveBrowser->isChecked() );
	config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
	fileOpenRecent->saveEntries(config,"Recent Files");

	config->setGroup("LASTDIRS");
#if KDE_IS_VERSION(3,1,3)
	config->writePathEntry("Extraction",extraction->getAccessPath());
	config->writePathEntry("Ouverture",archivechoice->getAccessPath());
#else
	config->writeEntry("Extraction",extraction->getAccessPath());
	config->writeEntry("Ouverture",archivechoice->getAccessPath());
#endif
}


void KarchiveurApp::readOptions()
{
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewToolbar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusbar->setChecked(bViewStatusbar);
  slotViewStatusBar();

  bool bViewNavigateur = config->readBoolEntry("Show Navigateur", true);
	viewArchiveBrowser->setChecked(bViewNavigateur);
	slotViewButineur();

  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);

  // initialize the recent file list
  fileOpenRecent->loadEntries(config,"Recent Files");

  QSize size=config->readSizeEntry("Geometry", new QSize(640,480));
	if( !size.isEmpty() )
	{
		resize(size);
	}

	config->setGroup("OPTIONS");
	includewholepath = config->readBoolEntry("ToutChemin",false);
	recursive = config->readBoolEntry("Recursif",true);
	overwritegzip = config->readBoolEntry("EcraserGzip",true);
	cheminabsolu = config->readBoolEntry("CheminAbsolu",false);
  view->setAlternateBackground( config->readBoolEntry("AlternateBackground",true) );
  viewbydirectories=config->readBoolEntry("ReadByDirs",true);
  preferedArchiveType=config->readEntry("PreferedArchiveType",".tar.gz");
  defaultdropbehaviour=config->readBoolEntry("DefaultDropBehaviour",false);

	switch(config->readNumEntry("LectureParFlot",1))
	{
		case 0:
			readarchivewithstream=false;//Many problems with stream reading, not needed anymore
			archiveobj->setDisplayIcons(true);
			break;
	 	case 1:
			readarchivewithstream=false;
			archiveobj->setDisplayIcons(true);
			break;
		case 2:
			//Don't display icons for archive dontent display.
			readarchivewithstream = false;
			archiveobj->setDisplayIcons(false);
			break;
	}

	if ( config->readNumEntry("SelectionMode",0) == 0 )
	{
		view->setSelectionMode(QListView::Multi);
	}
	else
	{
		view->setSelectionMode(QListView::Extended);
	}

	// For the compress rate, in the config file 1 = min compress
	compresslevel=config->readNumEntry("TauxCompression",9);
	archiveobj->setCompressRate(compresslevel);

	config->setGroup("LASTDIRS");
	QString cheminExtraction=config->readPathEntry("Extraction",QDir::homeDirPath());
	QString cheminOuverture=config->readPathEntry("Ouverture",QDir::homeDirPath());

	optionExtract=config->readNumEntry("OptionExtraction",1);
	optionOpening=config->readNumEntry("OptionOuverture",1);
	archivechoice->setFullPath(cheminOuverture,optionOpening);
	archivechoice->setForInitPreviousPath(cheminOuverture);
	navigateur->setPath(archivechoice->getAccessPath());
	extraction->setFullPath(cheminExtraction, optionExtract);

	config->setGroup("DATE");
	archiveobj->setKindOfDate(config->readNumEntry("KindOfDate",0));

	config->setGroup("TipOfTheDay");
	displaytipofday=config->readBoolEntry("show_tod", true);

	archiveobj->setRecursiveMode(recursive);
}


void KarchiveurApp::saveProperties(KConfig *)
{
}


void KarchiveurApp::readProperties(KConfig* _cfg)
{
	QString filename = _cfg->readPathEntry("filename");
	bool modified = _cfg->readBoolEntry("modified", false);

	if( modified )
	{
		bool canRecover;
		QString tempname = kapp->checkRecoverFile(filename, canRecover);

		if(canRecover)
		{
			CFileInfo info(filename);
			QFile::remove(tempname);
		}
	}
	else
	{
		if(!filename.isEmpty())
		{
		}
	}
}

bool KarchiveurApp::queryClose()
{
	return true;
}

bool KarchiveurApp::queryExit()
{
	saveOptions();
	return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KarchiveurApp::slotFileNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));

  KarchiveurApp *new_window= new KarchiveurApp();
  new_window->show();

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KarchiveurApp::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));
	setCaption("");

  view->clear();
  operation=NOARCHIVE;

	if(archiveobj!=NULL)
		delete archiveobj;
	archiveobj=NULL;

  slotAddFiles();
}

void KarchiveurApp::slotFileOpen()
{
  slotStatusMsg(i18n("Opening archive..."));
	led->setColor(QColor("red"));

	if(archivechoice->openArchive()==NULL)
	{
			slotStatusMsg(IDS_STATUS_DEFAULT);
			return;
	}

	displayArchiveContent();

  displayMessageArchiveSize();
}

void KarchiveurApp::slotFileOpenRecent(const KURL& url)
{
  QString archiveOpenName=openRemoteFile(url);
  if(!archiveOpenName.isEmpty())
    {
      archivechoice->setArchiveName(archiveOpenName);
      navigateur->setPath(archivechoice->getAccessPath());

      displayArchiveContent();
    }
}

void KarchiveurApp::slotFileOpenRecent(int id_)
{
  archivechoice->setArchiveName(recentFiles.at(recentFilesMenu->indexOf(id_)));
	navigateur->setPath(archivechoice->getAccessPath());
	displayArchiveContent();
}

/* This method will convert the archive format*/
void KarchiveurApp::slotFileSaveAs()
{
	led->setColor(QColor("red"));
	slotStatusMsg(i18n("Convert archive format..."));

	if( operation != NOARCHIVE )
	{
		QString repertoire;
		QString tmp=NULL;
		repertoire=reptmp;

		//We will extract the archive to a temp folder, so let's clean it
		process->clearArguments();
		*process << "rm" << "-fR" <<repertoire;
		process->start(KProcess::Block);

		process->clearArguments();
		*process << "mkdir" << reptmp;
		process->start(KProcess::Block);

		operation = CONVERSION;
		progress->reset();
		//progress.setTexte(i18n("Extracting files"));
		progress->setTotalSteps ( view->childCount() );
		archiveobj->extractArchive(reptmp,0,tmp);
		led->setColor(QColor("green"));
	}
	else
	{
		slotStatusMsg(i18n("First open an archive!"));
		led->setColor(QColor("orange"));
		return;
	}

	slotStatusMsg(IDS_STATUS_DEFAULT);
}

void KarchiveurApp::slotFileClose()
{
	slotStatusMsg(i18n("Closing file..."));
	close();
	//slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KarchiveurApp::slotFilePrint()
{
	slotStatusMsg(i18n("Printing..."));

	QPrinter printer;

	if (printer.setup(this))
	{
		view->print(&printer);
	}

	slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KarchiveurApp::slotFileQuit()
{
	slotStatusMsg(i18n("Exiting..."));
	saveOptions();
	close();
}

void KarchiveurApp::slotViewToolBar()
{
	slotStatusMsg(i18n("Toggle the toolbar..."));
	///////////////////////////////////////////////////////////////////
	// turn Toolbar on or off
	if(!viewToolbar->isChecked())
	{
		toolBar("mainToolBar")->hide();
	}
	else
	{
		toolBar("mainToolBar")->show();
    if(supportDisplayByDirs())
      cbdirectories->show();
	}

	slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KarchiveurApp::slotViewStatusBar()
{
	slotStatusMsg(i18n("Toggle the statusbar..."));
	///////////////////////////////////////////////////////////////////
	//turn Statusbar on or off
	if(!viewStatusbar->isChecked())
	{
		statusBar()->hide();
	}
	else
	{
		statusBar()->show();
    if(viewbydirectories)
      cbdirectories->show();
    else
      cbdirectories->hide();
	}
	repaint();
	slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

void KarchiveurApp::slotStatusMsg(const QString &text)
{
	///////////////////////////////////////////////////////////////////
	// change status message permanently
	statusBar()->clear();
	statusBar()->changeItem(text, ID_STATUS_MSG);
}


void KarchiveurApp::slotStatusHelpMsg(const QString &text)
{
	///////////////////////////////////////////////////////////////////
	// change status message of whole statusbar temporary (text, msec)
	statusBar()->message(text, 2000);
}


/* displays a window and search recursively  *
 * for archives in a directory               */
void KarchiveurApp::slotFileSearch()
{
	searchBox->setPath(archivechoice->getAccessPath(), optionOpening);
	searchBox->exec();
}

/** Make all or some files to be extracted */
void KarchiveurApp::slotExtractArchive()
{
	QString tmp=NULL;
	if(operation==NOARCHIVE)
	{
		slotStatusMsg(i18n("First open an archive!"));
		led->setColor(QColor("orange"));
		return;
	}

	slotStatusMsg(i18n("Extract files..."));
	led->setColor(KLed::red);

	extraction->displayPath();
	if(extraction->exec())
	{
		operation=EXTRACTING;
		archiveobj->extractArchive(extraction->getAccessPath(),extraction->extractAll(),tmp);
	}
	else
	{
		led->setColor(KLed::green);
		displayMessageArchiveSize();
	}
}

/** add some files to the archive */
void KarchiveurApp::slotAddFiles()
{
	CAddFilesToArchive ajoutfichiers;

	slotStatusMsg(i18n("Add files..."));
	led->setColor(QColor("red"));

	if(ajoutfichiers.exec()==0)
	{
		if(operation!=-1)
		{
			//An archive is already loaded
			operation=ADDING;
			QString archive;
			int actionfichiers;
			bool enleverfichiers;
			QStrList fichiersAjout;

			archive=archivechoice->getArchiveName();
			fichiersAjout=ajoutfichiers.getFichiers();
			actionfichiers=ajoutfichiers.getAction();
			enleverfichiers=ajoutfichiers.getRemoveFiles();

			addToArchive(archive,fichiersAjout,enleverfichiers,actionfichiers);
		}
		else
		{
			//No, so create an archive
			int kindofCompressor;
			CFileDialog boiteSauver(i18n("Create archive"),"*",this,"boite creer archive",false);

			//boiteSauver.setName(i18n("Create archive"));
			QString nomarchive=boiteSauver.getSaveFileName();
			archivechoice->setArchiveName(nomarchive);

			if(archivechoice->findKindOfCompressor()==1)
			{
				slotStatusMsg(i18n("Unknown archive type"));
				return;
			}
			operation=-2;
			kindofCompressor=archivechoice->getKindOfCompressor();
			if(strcmp(archivechoice->getProgrCompresseur(),"tar")==0)
			{
				progress->reset();
				progress->setTotalSteps(5);
				if(kindofCompressor==TARGZ)
					archiveobj=new CTarGz();
				else if(kindofCompressor==TARBZ2)
					archiveobj=new CTarBz2();
				else
					archiveobj=new CTar();
				//progress->setTexte(i18n("Creating archive"));
				progress->setProgress(progress->progress()+1);
				archiveobj->createArchive(nomarchive,ajoutfichiers.getFichiers());
				progress->setProgress(progress->progress()+1);
				displayArchiveContent();
			}
			else
			{
			  KMessageBox::sorry( this,
					      i18n("Files can be added only in "
						   ".tar, .tar.gz or .tar.bz2"),
					      "karchiver" );
			  led->setColor(QColor("orange"));
			  return;
			}
		}
    led->setColor(KLed::green);
    displayMessageArchiveSize();
	}
  else
  {
    if(operation==NOARCHIVE)
      led->setColor(KLed::red);
    else
      led->setColor(QColor("orange"));
  }

}

/** display a file using its associated program */
void KarchiveurApp::slotDisplayFile()
{
	slotStatusMsg(i18n("View this file..."));
	led->setColor(QColor("red"));

	if(operation!=NOARCHIVE)
	{
		QString txt;
		KFileOpenWithHandler* owh;
		operation=VISUALISATION;
		QListViewItem *li=view->currentItem();

		if(li==0)
		{
		  slotStatusMsg(i18n("First select a file"));
		  led->setColor(QColor("orange"));
		  return;
		}
		//Get the path of selected element
		txt=li->text(6);
		if(!txt.isEmpty())
		{
			txt=txt.remove(0,1);
			txt+=li->text(0);
		}
		else
		{
			txt=li->text(0);
			txt=txt.remove(0,1);
		}
		archiveobj->extractArchive(reptmp, 9,txt);
		waitpid(process->getPid(),NULL,WNOHANG);
		if((archivechoice->getKindOfCompressor()==GZIP)||(archivechoice->getKindOfCompressor()==BZIP2))
		{
			txt=txt.right(txt.length()-txt.findRev('/')-1);
		}
		txt=reptmp+txt;

		KFileItem fi(0,0,KURL(txt));
		owh=new KFileOpenWithHandler();
		fi.run();
	}

	slotStatusMsg(IDS_STATUS_DEFAULT);
	led->setColor(QColor("green"));
}

/** display all selected files */
void KarchiveurApp::slotDisplaySelectedFiles()
{
	slotStatusMsg(i18n("View all selected files..."));

	if(operation!=NOARCHIVE)
	{
		int i;
		int k;
		QListViewItem *li=view->firstChild();
		QString txt;
		KFileOpenWithHandler* owh;
		KFileItem *fi;

		if(li==0)
		{
		  slotStatusMsg(i18n("First select a file"));
		  led->setColor(QColor("orange"));
			return;
		}

		led->setColor(QColor("red"));

    owh=new KFileOpenWithHandler();
		k=view->childCount();

		for(i=0;i<k;i++)
		{
			if(view->isSelected(li))
			{
				txt=li->text(6);
				txt=txt.remove(0,1);
				txt+=li->text(0);
				archiveobj->extractArchive(reptmp, 9, txt);
				txt=reptmp+txt;
				fi=new KFileItem(0,0,KURL(txt));
				fi->run();
			}
			if(i!=k)
				li=li->nextSibling();
		}
	}

	led->setColor(QColor("green"));
  slotStatusMsg(IDS_STATUS_DEFAULT);
}

/** remove from archive all selected files */
void KarchiveurApp::slotRemoveSelectedFiles(){
  led->setColor(QColor("red"));
  slotStatusMsg(i18n("Deleting selected files..."));
  kdDebug()<<("IN:slotRemoveSelectedFiles\n");
	if(operation!=NOARCHIVE)
	{
		int i;
		int k;
		QListViewItem *li=view->firstChild();
		QString txt;
		QStrList strlist;

		k=view->childCount();
		for(i=0;i<k;i++)
		{
			if(view->isSelected(li))
			{
				txt=li->text(6);
				if(txt=="")
				{
					txt=li->text(0);
					txt=txt.remove(0,1);
				}
				else
				{
					txt=txt.remove(0,1);
					txt+=li->text(0);
				}
				strlist.append(txt);
				kdDebug()<<QString("slotRemove:%1*").arg(txt)<<endl;
			}
			if(i!=k)
				li=li->nextSibling();
		}
		if(KMessageBox::questionYesNoList(this, i18n("Do you really want to delete these files?"),
																						QStringList::fromStrList(strlist),
																						i18n("KArchiver"))
			==KMessageBox::No)
		{
			led->setColor(QColor("green"));
		  slotStatusMsg(IDS_STATUS_DEFAULT);
			return;
		}

		archiveobj->removeFilesFromArchive(strlist);
		operation=STANDBY;
	}

	displayArchiveContent();
	led->setColor(QColor("green"));
	displayMessageArchiveSize();
}

/** display some infos about the current archive */
void KarchiveurApp::slotDisplayArchiveInfos(){
	QListViewItem *item;
	QString txt;
	int total;
	int moyenne;
	int ecarttype;
	int taux;
	int taillearchive;
	int nbrefichiers;
	int i;

	if((archivechoice->getKindOfCompressor()==GZIP)||(archivechoice->getKindOfCompressor()==BZIP2)||(operation==NOARCHIVE))
		return;

	total=0;
	ecarttype=0;

	item=NULL;
	item=view->firstChild();
	nbrefichiers= view->childCount();
	while(item)
	{
		txt=item->text(1);
		i=txt.toInt();
		total+=i;
		ecarttype+=i*i/nbrefichiers;
		item=item->nextSibling();
	}

	taillearchive=archiveSize(archivechoice->getArchiveName());
	moyenne=total/nbrefichiers;
	taux=100-100*taillearchive/total;

	ecarttype-=moyenne;
	ecarttype=(int)sqrt((float)ecarttype);
	CInfos boiteinfos;
	boiteinfos.initialise(nbrefichiers, taillearchive, total, moyenne, taux, ecarttype);
	boiteinfos.exec();
}

/** allow user to set the options */
void KarchiveurApp::slotSetOptions(){
	CPreferences prefs;

	if(prefs.exec())
	{
		recursive=prefs.getRecursif();
		archiveobj->setRecursiveMode(recursive);
		cheminabsolu=prefs.getCheminAbsolu();
		overwritegzip=prefs.getOverwriteGzip();
		archiveobj->setOverwriteFilesWhenExtracting(overwritegzip);
		includewholepath=prefs.getWholePath();
		compresslevel=prefs.getCompressLevel();
		optionExtract=prefs.getKindofExtractionDirectory();
		optionOpening=prefs.getKindofOpeningDirectory();
		archiveobj->setIconSize(prefs.getSizeofIconListe());
		archivechoice->setOptionOpening(optionOpening);
		extraction->setOptionExtraction(optionExtract);
		view->setFont(prefs.getFont());
		archiveobj->setKindOfDate(prefs.getKindOfDate());
    view->setAlternateBackground(prefs.getAlternateBackground());
    viewbydirectories=prefs.getReadByDirs();
    preferedArchiveType=prefs.getPreferedArchiveType();
    defaultdropbehaviour=prefs.getDefaultDropBehaviour();

		switch(prefs.getLitParFlot())
		{
			case 0:
				readarchivewithstream=true;
				archiveobj->setDisplayIcons(true);
				break;
			case 1:
				readarchivewithstream=false;
				archiveobj->setDisplayIcons(true);
				break;
			case 2:
				readarchivewithstream=false;
				archiveobj->setDisplayIcons(false);
				break;
		}

		archiveobj->setCompressRate(prefs.getCompressLevel());
		if(prefs.getSelectionMode()==0)
			view->setSelectionMode(QListView::Multi);
		else
			view->setSelectionMode(QListView::Extended);
	}
}

/** Split/Unsplit the archive in/from small parts */
void KarchiveurApp::slotDiskCutUncut(){
	CWizardStep2Cutt wiz_cutt;

	if(wiz_cutt.exec())
	{
		if(wiz_cutt.rep!=NULL)
			doCuttUnCutt(wiz_cutt.rep);
	}
	return;
}

/* the archive browser has been close => update the configuration menu */
void KarchiveurApp::slotHideButineur() {
  viewArchiveBrowser->setChecked(false);
  slotViewButineur();
}

/** display or not the archive browser */
void KarchiveurApp::slotViewButineur(){
 if(viewArchiveBrowser->isChecked())
	{
		navigateur->show();
	}
	else
		navigateur->hide();
}

/** this method will launch the right compressor
to list the file contents
It will then launch the right method to display
the content in the listview */
void KarchiveurApp::displayArchiveContent()
{
	QString message;
	QString nom;
	int kindofCompressor;

	led->setColor(QColor("red"));

	if(archivechoice->findKindOfCompressor()==1)
	{
		slotStatusMsg(i18n("Unknown archive type"));
		operation=NOARCHIVE;
		led->setColor(QColor("orange"));
		return;
	}
	if(archivechoice->getKindOfCompressor()==9)
	{
		//This is a crypted archive
		CGPG crypt;
		kdDebug() << QString("Will decrypt %1").arg(archivechoice->getArchiveName()) << endl;
		crypt.setArchiveToProcess( archivechoice->getArchiveName());
		crypt.decryptArchive();
		archivechoice->setArchiveName(archivechoice->getArchiveName().remove(archivechoice->getArchiveName().length(),4));
  	if(archivechoice->findKindOfCompressor()==1)
   	{
   		slotStatusMsg(i18n("Unknown archive type"));
   		operation=NOARCHIVE;
   		led->setColor(QColor("orange"));
   		return;
   	}
	}

/*	if(archivechoice->isCurrentCompressorAvaible())
	{
		KMessageBox::sorry( this,
											i18n("The required compressor is not installed\non your system. See options->compressors to\ncheck which one you have!"),
											i18n("karchiver: compressor not found!")
											);
	}
*/
	nom=archivechoice->getArchiveName();
	setCaption(nom);

  kdDebug()<<QString("displayArchiveContent of %1").arg(nom);

  fileOpenRecent->addURL(KURL(archivechoice->getArchiveName()));
	navigateur->setPath(archivechoice->getCurrentArchiveDirectory());
	enableMenus(true);

	view->clear();
	for(int i=0;i<7;i++)
		view->setColumnWidth(i,50);
	operation=READING;

	kindofCompressor=archivechoice->getKindOfCompressor();

	if(archiveobj!=NULL)
		delete archiveobj;

	switch(kindofCompressor)
	{
		case TARGZ:
			archiveobj=new CTarGz();
			break;
		case TARBZ2:
			archiveobj=new CTarBz2();
			break;
		case TAR:
			archiveobj=new CTar();
			break;
		case ZIP:
			archiveobj=new CZip();
			break;
		case ARJ:
			archiveobj=new CArj();
			break;
		case LHA:
			archiveobj=new CLha();
			break;
		case RAR:
			archiveobj=new CRar();
			break;
		case GZIP:
			archiveobj=new CGz();
			break;
		case BZIP2:
			archiveobj=new CBz2();
			break;
	}
	connect(archiveobj, SIGNAL(archiveReadEnded()), this, SLOT(slotEndProcess()));

  archiveSetPassword->setEnabled(archiveobj->supportPassword());
	archiveobj->setReadArchiveWithStream(readarchivewithstream);
	archiveobj->setArchiveName(archivechoice->getArchiveName());
  archiveobj->setDisplayArchiveByDirectories(viewbydirectories);
  cbdirectories->clear();
	archiveobj->displayArchiveContent();
}

/** return the size in bytes of the current archive */
int KarchiveurApp::archiveSize(const QString & archive){
  QFileInfo file(archive);

  return file.size();
}

/** Launch the wizard */
void KarchiveurApp::slotWizard(){
	//First, we extract all files to reptmp
	//Once process exits to slotEndProcess, we launch the wizard
	if(operation!=-1)
	{
		QString repertoire;
		QString tmp=NULL;

		led->setColor(KLed::red);
		slotStatusMsg(i18n("Running wizard"));

		repertoire=reptmp;
		process->clearArguments();
		*process << "rm" << "-fR" <<repertoire;
		process->start(KProcess::Block);
		process->clearArguments();
		*process << "mkdir" << reptmp;
		process->start(KProcess::Block);

		operation=SCRIPT;
		archiveobj->extractArchive(reptmp,0,tmp);
	}
}

/** The current process ended */
void KarchiveurApp::finProcessus(KProcess*){
	int kindofCompressor=archivechoice->getKindOfCompressor();

	if(operation==READING)
	{
		led->setColor(QColor("green"));
		displayMessageArchiveSize();
		operation=STANDBY;
		return;
	}
	else if(operation==EXTRACTING)
	{
		if((kindofCompressor==GZIP)||(kindofCompressor==BZIP2))
		{
			fclose(fichierIO);
			led->setColor(KLed::green);
		}
		progress->setProgress(0);
		operation=STANDBY;
	}
	else if(operation==ADDCREATE)
		{
			displayArchiveContent();
			progress->setProgress(0);
			fclose(fichierIO);
		}
	else if(operation==VISUALISATION)
	{
		if((kindofCompressor==GZIP)||(kindofCompressor==BZIP2))
			fclose(fichierIO);
	}
	displayMessageArchiveSize();
}


/** we recive some informations through the standard
output of the process */
void KarchiveurApp::recueSdtOut(KProcess *, char *buffer, int length){

	if((operation==CONVERSION)||(operation==SCRIPT))
	{
		int i=0;
		if(archivechoice->getKindOfCompressor()==ZIP)
		{
			counter++;
			if(counter%2==0)
				progress->setProgress(progress->progress()+ 1 );
		}
		//A little animation for waiting gzip and bzip2 files to uncompress
		else if((archivechoice->getKindOfCompressor()==GZIP)||(archivechoice->getKindOfCompressor()==BZIP2))
		{
			counter++;
			fwrite(buffer,sizeof(char),length,fichierIO);
			QString s=i18n("Running compressor...");
  		switch(counter/4%8)
  		{
  			case 0:
  				s+="-";
  				break;
  			case 1:
  				s+="/";
  				break;
  			case 2:
  				s+="|";
  				break;
  			case 3:
  				s+="\\";
  				break;
  			case 4:
  				s+="-";
  				break;
  			case 5:
  				s+="/";
  				break;
  			case 6:
  				s+="|";
  				break;
  			case 7:
  				s+="\\";
  				break;
   		}
  		slotStatusMsg(s);
 		}
 		//For all other formats, use the progress bar
 		else
 		{
			QString inter;
			inter=buffer;
			inter=inter.left(length);

 			while((i=inter.find('\n',i+1))!=-1){
 				progress->setProgress(progress->progress()+1);
 			}
 		}
	}
	//We add files to an archive or we create one
	else if(operation==ADDCREATE)
	{
		if((archivechoice->getKindOfCompressor()==GZIP)||(archivechoice->getKindOfCompressor()==BZIP2))
		{
			fwrite(buffer,sizeof(char),length,fichierIO);
		}
		else
		{
			progress->setProgress(progress->progress()+1);
			//if(progress.estAnnule())
			//	process->kill();
		}
	}
	//We see a gzip or bzip2 archive
	else if(operation==VISUALISATION)
	{
		if((archivechoice->getKindOfCompressor()==GZIP)||(archivechoice->getKindOfCompressor()==BZIP2))
		{
			fwrite(buffer,sizeof(char),length,fichierIO);
		}
	}
}

/** Add some files to the archive
@fichiersAjout : list of files to add
@enlevefichiers : remove or not those files from disk
@actionfichiers : 0=mode append and replace files, 1=mode update files
@cheminRelatif : include only filenames, without their path */
void KarchiveurApp::addToArchive(const QString &, QStrList fichiersAjout, bool enlevefichiers, int actionfichiers,const char* cheminRelatif){
	archiveobj->addFilesToArchive( fichiersAjout,enlevefichiers,actionfichiers,cheminRelatif);
	displayArchiveContent();
}

/** Compress with the compressor compresseur the archive nomArchive*/
void KarchiveurApp::compressCreatedArchive(const QString & compresseur, const QString & nomArchive){
	process->clearArguments();
	//gzip & bzip2 accept -f
	*process << compresseur;
	if(((strcmp(compresseur,"gzip")==0)&&overwritegzip)||((strcmp(compresseur,"bzip2")==0)&&overwritegzip))
		*process << "-f";
	*process << nomArchive;
	process->start(KProcess::Block);
}

/** Remove the files fichiersAEffacer from archive archive */
void KarchiveurApp::removeFilesFromArchive(const QString &, QStrList fichiersAEffacer){
	if(operation==NOARCHIVE)
	{
		led->setColor(QColor("orange"));
		return;
	}

	led->setColor(QColor("red"));

	archiveobj->removeFilesFromArchive(fichiersAEffacer);

	kdDebug()<<("Will display archiveliste\n");
	displayArchiveContent();
	kdDebug()<<("removeFilesFromArchive:OK\n")<<endl;
	led->setColor(QColor("green"));
}

/** Open the archive nomArchive, which was selected
from the archive browser */
void KarchiveurApp::slotOpenFileFromArchiveBrowser(const QString& nomArchive){
	QString str;

  slotStatusMsg(i18n("Opening archive..."));
	led->setColor(QColor("red"));

  str=navigateur->getChemin();//archivechoice->getCurrentArchiveDirectory();
//	str+="/";
	str+=nomArchive;
	archivechoice->setArchiveName(str);
  kdDebug()<<QString("slotOpenFileFromArchiveBrowser: opening %1").arg(str)<<endl;

	displayArchiveContent();
  displayMessageArchiveSize();
}

/** Open the archive nomArchive, which was found
using the search tool */
void KarchiveurApp::slotOpenFileFromArchiveFinder(const QString& nomArchive){
	QString nom;
	led->setColor(KLed::red);

	nom=nomArchive;
	archivechoice->setFullPath(nom.left(nom.findRev('/')));
	navigateur->setPath(archivechoice->getAccessPath());
	archivechoice->setArchiveName(nomArchive);
	displayArchiveContent();
}

/** Open the archive nomArchive, which was sent
by the command line */
void KarchiveurApp::openFileFromCommandLine(QStrList names){
  openFilesDropPaste(QStringList::fromStrList(names));
}

/** Convert an archive to a new one; prompt for a filename, & guess the mimetype*/
void KarchiveurApp::doArchiveConversion(){
  QString baserep;
  QString newarchivename;
  QString oldarchivename;
  CArchiveChoice archivemanager;
  CFileInfo fileinfo;
  QString filter;

  fileinfo.setFile(archivechoice->getArchiveName());
  CFileDialog newarchivenamedlg(extraction->getAccessPath(),NULL,0,"convert_dialog",false,new QLabel(i18n("Choose the destination format of the archive !"),0));
  newarchivenamedlg.setSelection(fileinfo.baseName()+preferedArchiveType);
  newarchivenamedlg.setCaption(i18n("Convert an archive"));

  if(newarchivenamedlg.exec())
  {
    newarchivename=newarchivenamedlg.selectedFile();
    newarchivenamedlg.fixFileExtension(newarchivename);
  }

  if(newarchivename.isEmpty())
  {
    operation=STANDBY;
    led->setColor(QColor("orange"));
    return;
  }
  //Guess the mimetype
  archivemanager.setArchiveName(newarchivename);
  if(archivemanager.findKindOfCompressor()==1)
  {
    slotStatusMsg(i18n("Unknown archive type"));
    led->setColor(QColor("orange"));
    return;
  }
  oldarchivename=archivechoice->getArchiveName();
  archivechoice->setArchiveName(newarchivename);
  fileinfo.setFile(newarchivename);
  baserep=fileinfo.dirPath(true)+"/";
  operation=NOARCHIVE;

  kdDebug()<<QString("Conversion of %1, type=%2\n").arg(fileinfo.dirPath(true)).arg(archivemanager.getKindOfCompressor());
  doArchiveConversion(archivemanager.getKindOfCompressor(),baserep,oldarchivename);
}

/** Convert an archive to type, and store it in the directory repconv */
void KarchiveurApp::doArchiveConversion(int type, const QString & repconv, QString previousname){
  int kindofCompressor;
  QStrList listeFichiers;
  QStrList listeKDE;
  QString s;
  QString archiveOriginale;
  QString newarchivename;
  QDir repertoire=QDir::root();

  repertoire.cd(reptmp);
  if(previousname.isNull())
    archiveOriginale=archivechoice->getArchiveName();
  else
    archiveOriginale=previousname;

  listeFichiers=repertoire.encodedEntryList();
  for (QString f = listeFichiers.first(); f; f = listeFichiers.next())
  {
    if((strcmp(f,".")==0)||(strcmp(f,"..")==0))
      listeFichiers.remove();
    else
      listeKDE.append(f);
  }

  kdDebug()<<QString("Analysis of:%1, future type:%2,target dir:%3*").arg(archivechoice->getArchiveName()).arg(type).arg(repconv)<<endl;
  //We remove the extension of the archive name not needed anymore
  CFileInfo basename(archivechoice->getArchiveName());
  newarchivename=repconv+"/"+basename.baseName();

  //Final file name, once converted
	switch(type)
	{
		case 0:
      newarchivename+=".tar";
      archiveobj=new CTar();
      break;
		case 1:
      newarchivename+=".tar.gz";
      archiveobj=new CTarGz();
      break;
		case 2:
      newarchivename+=".tar.bz2";
      archiveobj=new CTarBz2();
      break;
		case 3:
      newarchivename+=".zip";
      archiveobj=new CZip();
      break;
		case 4:
      newarchivename+=".gz";
      break;
		case 5:
      newarchivename+=".bz2";
      break;
		case 6:
      newarchivename+=".rar";
      archiveobj=new CRar();
      break;
    case 7:
      newarchivename+=".lha";
      archiveobj=new CLha();
      break;
	}

	operation=-2;
	archivechoice->setArchiveName(newarchivename);
  archivechoice->findKindOfCompressor();
  kindofCompressor=archivechoice->getKindOfCompressor();

	if(strcmp(archivechoice->getProgrCompresseur(),"tar")==0)
    archiveobj->createArchive(newarchivename,listeKDE,reptmp);
	else if(kindofCompressor==ZIP)
		archiveobj->createArchive(newarchivename,listeKDE,reptmp);
	else if(kindofCompressor==RAR)
		archiveobj->createArchive(newarchivename,listeKDE,reptmp);
	else if(kindofCompressor==LHA)
		archiveobj->createArchive(newarchivename,listeKDE,reptmp);
	else
	{
		// We convert a .gz or .bz2 archive
		newarchivename=basename.dirPath()+"/"+basename.baseName();

		process->clearArguments();
		QString pr;
		QString ext;
		if(basename.extension()=="gz")
		{
			kindofCompressor=GZIP;
			pr="gzip";
			ext=".gz";
		}
		else if(basename.extension()=="bz2")
		{
			kindofCompressor=BZIP2;
			pr="bzip2";
			ext=".bz2";
		}
		else
		{
	  	KMessageBox::sorry( this,
			i18n("Archives can only be converted in "
			".tar, .tar.gz, .tar.bz2, .zip, .rar or .lha"), "karchiver" );
			led->setColor(QColor("orange"));
			return;
		}
    *process << pr;
    *process << "-f";
    *process << newarchivename;
    process->start(KProcess::Block);
    process->clearArguments();
    pr="mv";
    *process << pr;
    newarchivename+=ext;
    *process << newarchivename;
    *process << repconv;
    process->start(KProcess::Block);
	}
	s=i18n("Delete : %1?").arg(archiveOriginale);
	if(KMessageBox::questionYesNo(this, s, "karchiver")==1)
	{
		process->clearArguments();
		*process << "rm" << "-f" <<archiveOriginale;
		process->start(KProcess::Block);
	}
	led->setColor(QColor("green"));
  displayArchiveContent();
}

/** split or unsplit the archive nom */
void KarchiveurApp::doCuttUnCutt(QString nom, int blocksize){
	led->setColor(QColor("red"));

	cuttFichier objCutt;
	QString str;
  CFileInfo cutt(nom);
  CFileInfo uncutt(archivechoice->getArchiveName());

	if(cutt.extension()=="01")
	{
		nom=cutt.dirPath(true)+"/"+cutt.baseName();//nom.left(nom.length()-3);
		if(objCutt.Begin(nom,1)==0)
		{
			operation=-1;
			archivechoice->setArchiveName(nom);
			displayArchiveContent();
		}
	}
	else
	{
		//Not needed anymorestr=archivechoice->getArchiveName();
  	//	str=str.right(str.length()-str.findRev('/')-1);
		if(nom.right(1)!="/")
			nom+="/";
   //str=nom+uncutt.fileName();//str;

    objCutt.Begin(nom+uncutt.fileName(),0,blocksize, archivechoice->getArchiveName());

    nom=i18n("Archive split in %1.01...").arg(archivechoice->getArchiveName());
    KMessageBox::information(this,nom,i18n("KArchiver"));
	}
	led->setColor(QColor("green"));
}

/** Download a net file and returns its local filename*/
QString KarchiveurApp::openRemoteFile( const KURL& url){
	QString fichierlocal;

  fichierlocal=QDir::homeDirPath();
  fichierlocal+="/";
  fichierlocal+=url.fileName(true);
  kdDebug()<<QString("local file:%1*%2*").arg(fichierlocal).arg(url.prettyURL())<<endl;
  if ( url.isMalformed() )
  {
      KMessageBox::error(this, i18n("Bad URL!"));
      led->setColor(QColor("orange"));
      return NULL;
  }

  if (KIO::NetAccess::download(url, fichierlocal) == false)
  {
    // this needs to be handled within KIO sooooon
    KMessageBox::error(this, i18n("Cannot download file!"));
    led->setColor(QColor("orange"));
    return NULL;
  }

  return fichierlocal;
}

/** Displays in the status bar the size of the archive */
void KarchiveurApp::displayMessageArchiveSize(){
	QString message;

 	message=IDS_STATUS_DEFAULT;
 	if(archivechoice->getArchiveName()!=NULL)
	  message=QString("%1  -  %2 %3").arg(IDS_STATUS_DEFAULT).arg(archiveSize(archivechoice->getArchiveName())).arg(i18n("bytes"));
   slotStatusMsg(message);
}

/** A drop will occur, so we have to allow it */
void KarchiveurApp::dragEnterEvent(QDragEnterEvent* e){
  e->accept(QUriDrag::canDecode(e));
}

/** someone has dropped something ! */
void KarchiveurApp::dropEvent( QDropEvent *drop){
  QStringList strliste;

	QUriDrag::decodeToUnicodeUris( drop, strliste);

  openFilesDropPaste(strliste);

}

/** Receive a message when the user try to
	* drag something. It extract the selected files,
	* and then transmit their path to the target
	* application
 	*/
void KarchiveurApp::slotDoDragAndDrop(){
	QListViewItem* vi;
	QStrList urlsatransmettre;
	QString fichiersaextraire;
	QString repulterieur;
	QString tmp=NULL;
  int k;


  if(operation==NOARCHIVE)
  	return;

 	//We will extract the archive to a temp folder, so let's clean it
 	process->clearArguments();
 	*process << "rm" << "-fR" <<reptmp;
 	process->start(KProcess::Block);
 	process->clearArguments();
 	*process << "mkdir" << reptmp;
 	process->start(KProcess::Block);

	//Extract only the selected files
	archiveobj->extractArchive(reptmp, 1,tmp);

 	vi=view->firstChild();
 	for(k=0;k<view->childCount();k++)
 	{
 		if((vi->isSelected())&&(strcmp(vi->text(0),"..")!=0)&&repulterieur!=vi->text(6))
 		{
 			repulterieur=vi->text(6);
 			fichiersaextraire+=vi->text(6);
			if(vi->text(6).isEmpty()||vi->text(6).isNull()||vi->text(6)==" ")
	 			fichiersaextraire+=vi->text(0);
 			fichiersaextraire=fichiersaextraire.right(fichiersaextraire.length()-1);
 			fichiersaextraire=reptmp+fichiersaextraire;
			kdDebug()<<QString("*Drag of:%1*").arg(fichiersaextraire)<<endl;
 			urlsatransmettre.append(fichiersaextraire);
 			fichiersaextraire="";
 		}
 		vi=vi->nextSibling();
 	}
	QDragObject *d = new QUriDrag( urlsatransmettre, this );
	d->dragCopy();
}

/** The contextual menu has been activated */
void KarchiveurApp::slotContextualMenuActivated(int i){
	switch(contextualMenu->indexOf(i))
	{
		case 0:
			slotExtractArchive();
			break;
		case 2:
			slotCopy();
			break;
		case 3:
			slotPaste();
			break;
		case 4:
			slotRemoveToTrashSelectedFiles();
			break;
		case 5:
			slotRemoveSelectedFiles();
			break;
		case 7:
			slotDisplayFile();
			break;
		case 8:
			slotDisplayFileProprieties();
			break;
	}
}

/* The user right-clicked on a file in the listview
	* so we'll display a contextual menu */
void KarchiveurApp::slotShowContextualMenu ( QListViewItem *l, const QPoint &pos, int ){
	if(operation==NOARCHIVE)
		return;

	view->setSelected(l, true);
////	contextualMenu->exec(pos);
 QPopupMenu *pop = (QPopupMenu *)factory()->container("popup_list", this);
 pop->popup(pos);
}

/** Display in a dialog box some informations
concerning the selected file */
void KarchiveurApp::slotDisplayFileProprieties(){
	CProprietes propr;
	QListViewItem* l;
	int kindofCompressor;

	l=view->currentItem();
	kindofCompressor=archivechoice->getKindOfCompressor();

	propr.setFileName(l->text(0));
	propr.setFilePath(l->text(6));
	propr.setSize(l->text(1));
	if((kindofCompressor==TAR)||(kindofCompressor==TARGZ)||(kindofCompressor==TARBZ2))
		propr.setPermission(l->text(5));

	propr.exec();
}

/** The mouse button was clicked on the listview */
void KarchiveurApp::slotMouseClick(QListViewItem* e){
	if(operation==READING)
	{
		if(KMessageBox::questionYesNo(this, i18n("Stop opening of current archive?"), i18n("KArchiver")) == KMessageBox::Yes)
			archiveobj->stopReading();
	}
	else 	if(operation==EXTRACTING)
	{
		if(KMessageBox::questionYesNo(this, i18n("Stop extracting current archive?"), i18n("KArchiver")) == KMessageBox::Yes)
			archiveobj->stopExtracting();
	}
  else if(supportDisplayByDirs() && viewbydirectories && (archiveobj!=NULL)&&(e!=NULL))
    archiveobj->displayArchiveContent(e->text(0), e->text(6));
}

/** The user selected paste */
void KarchiveurApp::slotPaste(){
	QStringList urls;

	if ( QUriDrag::decodeToUnicodeUris(QApplication::clipboard()->data(), urls) )
		openFilesDropPaste(urls);
}

/** Some files were dropped or pasted; they have
been decoded in strliste, so just open them ! */
void KarchiveurApp::openFilesDropPaste( QStringList strliste ){
  QStrList strlistWithoutPath;	//Contains the names WITHOUT their path of dropped files
  QStrList strlistWithPath;	//Contains the names WITH their path of dropped files
  QString str;
  QString chemin;
  CAddFiles dlgTypeAction;
  QString archivename;
  CFileInfo fileinfo;
  int kindofCompressor;

  strlistWithoutPath.clear();
  strlistWithPath.clear();

  for ( QStringList::Iterator it=strliste.begin(); it!=strliste.end(); ++it)
  {
    kdDebug()<<QString("Drop:%1*").arg(*it)<<endl;
    str=openRemoteFile(KURL((*it)));
    strlistWithPath.append(str);
    //Let's keep only the filename
    fileinfo.setFile(str);
    strlistWithoutPath.append(fileinfo.fileName());
    chemin=(*it);
  }

  //The path, useful for the QStrList without path
  fileinfo.setFile(chemin);
  chemin=fileinfo.dirPath()+"/";
  chemin=chemin.right(chemin.length()-chemin.find(':')-1);
  archivechoice->setFullPath(chemin);
  navigateur->setPath(archivechoice->getAccessPath());
  kdDebug()<<QString("path:%1 *operation*%2 *").arg(chemin).arg(operation)<<endl;

  //We'll check if we have dragged only one file: a valid archive
  archivename=archivechoice->getArchiveName(); //Backup current archive name

  if(operation==NOARCHIVE)
    dlgTypeAction.canAddToCurrentArchive(false);

  if(strlistWithPath.count()==1)
  {
      //We have dropped one file and it is an archive, so modify the dialog box
      kdDebug()<<("Archive dropped\n");
      archivechoice->setArchiveName(strlistWithPath.first());
      if(archivechoice->findKindOfCompressor()==0)
      {
        dlgTypeAction.setCheckBoxArchive();
        operation=STANDBY;
        displayArchiveContent();
        return;
      }
      else
        archivechoice->setArchiveName(archivename);
   }
   //Display a default archive name, based on the first dropped file
   str=strlistWithPath.first();
   if(str.endsWith("/"))
     str.truncate(str.length()-1);
   dlgTypeAction.setDefaultArchiveName(str+preferedArchiveType);

  if((!defaultdropbehaviour)&&(!dlgTypeAction.exec()))
  {
    led->setColor(QColor("orange"));
    operation=STANDBY;
    return;
  }
  switch(defaultdropbehaviour ? 0 : dlgTypeAction.getAddFilesChoice())
    {
    case 0:
      //Add to this archive
      archivechoice->setArchiveName(archivename);
      archivechoice->findKindOfCompressor();
      operation=STANDBY;
      break;
    case 1:
      //Add to another archive
      archivechoice->setArchiveName(dlgTypeAction.getOtherArchiveName());
      archivechoice->findKindOfCompressor();
      kdDebug()<<QString("Add to archive:%1").arg(dlgTypeAction.getOtherArchiveName())<<endl;
      if(archiveobj!=NULL)
        delete archiveobj;

      switch(archivechoice->getKindOfCompressor())
      {
        case TARGZ:
          archiveobj=new CTarGz();
          break;
        case TARBZ2:
          archiveobj=new CTarBz2();
          break;
        case TAR:
          archiveobj=new CTar();
          break;
        case ZIP:
          archiveobj=new CZip();
          break;
        case ARJ:
          archiveobj=new CArj();
          break;
        case LHA:
          archiveobj=new CLha();
          break;
        case RAR:
          archiveobj=new CRar();
          break;
        case GZIP:
          archiveobj=new CGz();
          break;
        case BZIP2:
          archiveobj=new CBz2();
          break;
        default:
          led->setColor(QColor("orange"));
          operation=NOARCHIVE;
          return;
          break;
      }
      connect(archiveobj, SIGNAL(archiveReadEnded()), this, SLOT(slotEndProcess()));
      archiveobj->setReadArchiveWithStream(readarchivewithstream);
      archiveobj->setArchiveName(archivechoice->getArchiveName());
      operation=STANDBY;
      break;
    case 2:
      //Create a new archive
      archivechoice->setArchiveName(dlgTypeAction.getNewArchiveName());
      archivechoice->findKindOfCompressor();
      kdDebug()<<QString("Add to this new archive:%1").arg(dlgTypeAction.getNewArchiveName())<<endl;
      operation=NOARCHIVE;
      break;
    case 3:
      //Open current archive
      operation=STANDBY;
      displayArchiveContent();
      return;
      break;
    case -1:
      led->setColor(QColor("orange"));
      return;
      break;
    }

  archivename=archivechoice->getArchiveName();

  if(operation!=NOARCHIVE)
  {
    //An archive is already loaded

    kindofCompressor=archivechoice->getKindOfCompressor();

    if((kindofCompressor==TAR)||(kindofCompressor==TARGZ)||(kindofCompressor==TARBZ2))
    {
      operation=ADDING;
      if(includewholepath)
        addToArchive(archivename,strlistWithPath,FALSE,0,chemin);
      else
        addToArchive(archivename,strlistWithoutPath,FALSE,0,chemin);
    }
    else if((kindofCompressor==ZIP)||(kindofCompressor==LHA)||(kindofCompressor==RAR))
    {
      operation=ADDING;
      addToArchive(archivename,strlistWithPath,FALSE,0,chemin);
    }
    else
    {
      KMessageBox::error( this, i18n("Files can be added only in\n.tar, .tar.gz, .tar.bz2 .zip .rar .lha"), "karchiver" );
      led->setColor(QColor("orange"));
      return;
    }
  }
  else
  {
      //We will create an archive
      led->setColor(QColor("red"));

      if(archivename.isEmpty())
      {
        slotStatusMsg(i18n("Archive not created"));
        led->setColor(QColor("orange"));
        return;
      }
      archivechoice->setArchiveName(archivename);
      if(archivechoice->findKindOfCompressor()==1)
      {
        slotStatusMsg(i18n("Unknown archive type"));
        led->setColor(QColor("orange"));
        return;
      }
      navigateur->setPath(archivechoice->getAccessPath());
      kindofCompressor=archivechoice->getKindOfCompressor();
      ///		operation=AJOUTCREATION;
      if(strcmp(archivechoice->getProgrCompresseur(),"tar")==0)
      {
        switch(kindofCompressor)
        {
          case TARGZ:
            archiveobj=new CTarGz();
            break;
          case TARBZ2:
            archiveobj=new CTarBz2();
            break;
          case TAR:
            archiveobj=new CTar();
            break;
        }
        kdDebug()<<QString("Will create Tar based with rel.path:%1*").arg(chemin)<<endl;
        if(includewholepath)
          archiveobj->createArchive(archivename,strlistWithPath,chemin);
        else
          archiveobj->createArchive(archivename,strlistWithoutPath,chemin);
        operation=ADDCREATE;
        displayArchiveContent();
      }
      else if(strcmp(archivechoice->getProgrCompresseur(),"unzip")==0)
      {
        archiveobj=new CZip();
        kdDebug()<<QString("Will create Zip based with rel.path:%1*").arg(chemin)<<endl;
        if(includewholepath)
          archiveobj->createArchive(archivename,strlistWithPath,chemin);
        else
          archiveobj->createArchive(archivename,strlistWithoutPath,chemin);
        displayArchiveContent();
      }
      else if((strcmp(archivechoice->getProgrCompresseur(),"gzip")==0)&&(strlistWithPath.count()==1))
      {
        fichierIO=fopen(archivename,"w");
        kindofCompressor=GZIP;
        process->clearArguments();
        *process << "gzip" << "-c" << str;
        process->start(KProcess::NotifyOnExit, KProcess::Stdout);
      }
      else if((strcmp(archivechoice->getProgrCompresseur(),"bzip2")==0)&&(strliste.count()==1))
      {
        fichierIO=fopen(archivename,"w");
        kindofCompressor=BZIP2;
        process->clearArguments();
        *process << "bzip2" << "-c" << str;
        process->start(KProcess::NotifyOnExit, KProcess::Stdout);
      }
      else
      {
        KMessageBox::error( this,i18n("Files can be added only in\n .tar, .tar.gz, .tar.bz2"), "karchiveur" );
        led->setColor(QColor("orange"));
        return;
      }
      led->setColor(KLed::green);
    }
}

/** The user try to copy a karchiveur's file to
the clipboard */
void KarchiveurApp::slotCopy(){
  QListViewItem* vi;
  QStrList urlsatransmettre;
  QString fichiersaextraire;
  QString repulterieur;
  QString tmp=NULL;
  int k;

  if(operation==NOARCHIVE)
    return;

 	//We will extract the archive to a temp folder, so let's clean it
 	process->clearArguments();
 	*process << "rm" << "-fR" <<reptmp;
 	process->start(KProcess::Block);
 	process->clearArguments();
 	*process << "mkdir" << reptmp;
 	process->start(KProcess::Block);

	//Extract only the selected files
	archiveobj->extractArchive(reptmp, 1,tmp);

 	vi=view->firstChild();
 	for(k=0;k<view->childCount();k++)
 	{
 		if((vi->isSelected())&&(strcmp(vi->text(0),"..")!=0)&&repulterieur!=vi->text(6))
 		{
 			repulterieur=vi->text(6);
 			fichiersaextraire+=vi->text(6);
			if(vi->text(6).isEmpty()||vi->text(6).isNull()||vi->text(6)==" ")
	 			fichiersaextraire+=vi->text(0);
 			fichiersaextraire=fichiersaextraire.right(fichiersaextraire.length()-1);
 			fichiersaextraire=reptmp+fichiersaextraire;
//	 			fichiersaextraire+=vi->text(0);
 			urlsatransmettre.append(fichiersaextraire);
			kdDebug()<<QString("In copy:addURL:%1*").arg(fichiersaextraire)<<endl;
 			fichiersaextraire="";
 		}
 		vi=vi->nextSibling();
 	}

	kapp->clipboard()->setData( new QUriDrag( urlsatransmettre, this ));

}

/** extract the selected files to trash and then
remove them from the archive */
void KarchiveurApp::slotRemoveToTrashSelectedFiles(){
  slotStatusMsg(i18n("Move all selected files to trash..."));

	if(operation!=NOARCHIVE)
	{
		int i;
		int k;
		QListViewItem *li=view->firstChild();
		QString txt;
		QString repTrash;
		KConfig conf("kdeglobals"); //To get the "trash" directory

		if(li==0)
		{
		  slotStatusMsg(i18n("First select a file"));
		  led->setColor(QColor("orange"));
			return;
		}

		led->setColor(QColor("red"));
		repTrash=QDir::homeDirPath();
		repTrash+="/Desktop/Trash";
		conf.setGroup("Paths");
		repTrash=conf.readPathEntry("Trash",repTrash);
		k=view->childCount();

		for(i=0;i<k;i++)
		{
			if(view->isSelected(li))
			{
				txt=li->text(6);
				txt=txt.remove(0,1);
				txt+=li->text(0);
				archiveobj->extractArchive(repTrash, 9,txt);
			}
			if(i!=k)
				li=li->nextSibling();
		}
	}

	slotRemoveSelectedFiles();

	led->setColor(QColor("green"));
  slotStatusMsg(IDS_STATUS_DEFAULT);
}

/** the user has "cutted" some files in the current
archive */
void KarchiveurApp::slotCutt(){
	QListViewItem* vi;
	QStrList urlsatransmettre;
	QString fichiersaextraire;
	QString repulterieur;
	QString tmp=NULL;
  int k;

  kdDebug()<<("IN:slotCutt\n");
  if(operation==NOARCHIVE)
  	return;

 	//We will extract the archive to a temp folder, so let's clean it
 	process->clearArguments();
 	*process << "rm" << "-fR" <<reptmp;
 	process->start(KProcess::Block);
 	process->clearArguments();
 	*process << "mkdir" << reptmp;
 	process->start(KProcess::Block);

	//Extract only the selected files
	archiveobj->extractArchive(reptmp, 1,tmp);

 	vi=view->firstChild();
 	for(k=0;k<view->childCount();k++)
 	{
 		if((vi->isSelected())&&(strcmp(vi->text(0),"..")!=0)&&repulterieur!=vi->text(6))
 		{
 			repulterieur=vi->text(6);
 			fichiersaextraire+=vi->text(6);
			if(vi->text(6).isEmpty()||vi->text(6).isNull()||vi->text(6)==" ")
	 			fichiersaextraire+=vi->text(0);
 			fichiersaextraire=fichiersaextraire.right(fichiersaextraire.length()-1);
 			fichiersaextraire=reptmp+fichiersaextraire;
//	 			fichiersaextraire+=vi->text(0);
 			urlsatransmettre.append(fichiersaextraire);
			kdDebug()<<QString("In cutt:addURL:%1*").arg(fichiersaextraire)<<endl;
 			fichiersaextraire="";
 		}
 		vi=vi->nextSibling();
 	}
	QDragObject *d = new QUriDrag( urlsatransmettre, this );

	kapp->clipboard()->setData( d);

	KMessageBox::information(this, i18n("Paste those files in any application NOW!"), "karchiveur");
	slotRemoveSelectedFiles();

}

/** select all the files in the current archive */
void KarchiveurApp::slotSelectAll(){
	view->selectAll(true);
}

/** unselect all files in the current archive */
void KarchiveurApp::slotUnselectAll(){
	view->selectAll(false);
}

/** find in this archive all items matching the
name name */
void KarchiveurApp::slotFindInThisArchive( QString name ){
	QListViewItem *vi;
	int k;

	kdDebug()<<QString("Find:%1*%2*").arg(name).arg(indexfind)<<endl;
 	vi=view->firstChild();
 	for(k=0;k<view->childCount();k++)
 	{
 		if((k>indexfind)&&(vi->text(0).contains(name)!=0))
 		{
			view->setCurrentItem(vi);
			vi->setSelected(true);
			indexfind=k;
			return;
 		}
 		vi=vi->nextSibling();
 	}

 	KMessageBox::information(this, i18n("No more occurrences found!"), "karchiveur");
 	indexfind=-1;
}
/** the user closed the find in this archive tool */
void KarchiveurApp::slotCloseFind(){
	find->hide();
	indexfind=-1;
}

/** Use the find in this archive tool */
void KarchiveurApp::slotUseFind(){
	find->show();
	indexfind=-1;
}

void KarchiveurApp::slotSetPassword(){
	 QCString pass=archiveobj->getPassword();
//   KPasswordDialog *dialog=new KPasswordDialog(KPasswordDialog::Password,true,0,NULL,"X");
//   dialog->addLine(i18n("validy duration"),i18n("Attention: this password is during all operations on this zip archive\nuntil you change or delete it\nor a new archive is opened."));
//   dialog->addLine(i18n("security"),i18n("Note: this operation is not secure; other people may read your password and/or can easy hack the archive!"));
//   dialog->setPrompt(i18n("Enter Archive password"));
     int rc = KPasswordDialog::getPassword(pass,i18n("Enter Archive password"));
     if(rc==KPasswordDialog::Accepted)
        archiveobj->setPassword(pass);
//   dialog->show();
 //  dialog->~KPasswordDialog();
return;

}
/** Remove form current archive the selected fies,
with an undo option (i.e. extract to reptmpundo, then
remove) */
void KarchiveurApp::slotRemoveWithUndoSelectedFiles(){
  slotStatusMsg(i18n("Deleting selected files..."));

	if(operation!=NOARCHIVE)
	{
		int i;
		int k;
		QListViewItem *li=view->firstChild();
		QString txt;
		QString repTrash;

		if(li==0)
		{
		  slotStatusMsg(i18n("First select a file"));
		  led->setColor(QColor("orange"));
			return;
		}

		led->setColor(QColor("red"));

		k=view->childCount();
		listUndo.clear();
		listRedo.clear();

		for(i=0;i<k;i++)
		{
			if(view->isSelected(li))
			{
				txt=li->text(6);
				txt=txt.remove(0,1);
				txt+=li->text(0);
				listUndo.append(txt);
				listRedo.append(txt);
				archiveobj->extractArchive(reptmpundo, 9, txt);
			}
			if(i!=k)
				li=li->nextSibling();
		}
	}

	slotRemoveSelectedFiles();

	led->setColor(QColor("green"));
  slotStatusMsg(IDS_STATUS_DEFAULT);
}

/** Undo the previous operation, i.e. the deletion
od files from archive */
void KarchiveurApp::slotUndo(){
	if(listUndo.isEmpty())
	{
		led->setColor(QColor("orange"));
		slotStatusMsg(i18n("Nothing to undo!"));
		return;
	}

	addToArchive(archivechoice->getArchiveName(), listUndo, true, 0, reptmpundo);
	listUndo.clear();
}

/** Redo the last operation (i.e. remove once
again the selected file) */
void KarchiveurApp::slotRedo(){
	if((listRedo.isEmpty())||(!listUndo.isEmpty()))
	{
		led->setColor(QColor("orange"));
		slotStatusMsg(i18n("Nothing to redo!"));
		return;
	}

	slotRemoveWithUndoThoseFiles(listRedo);
	listRedo.clear();
}

/** Remove from current archives the files in
files, with undo possibility */
void KarchiveurApp::slotRemoveWithUndoThoseFiles( QStrList files ){
  slotStatusMsg(i18n("Deleting selected files..."));

	if(operation!=NOARCHIVE)
	{
		QString txt;
		QString repTrash;

		led->setColor(QColor("red"));

		listUndo.clear();
		listRedo.clear();

		for(txt=files.first(); txt; txt=files.next())
		{
				listUndo.append(/*reptmpundo+"/"+*/txt);
				listRedo.append(txt);
				archiveobj->extractArchive(reptmpundo, 9,txt);
		}
	}

	slotRemoveThoseFiles(files);

	led->setColor(QColor("green"));
  slotStatusMsg(IDS_STATUS_DEFAULT);
}

/** Remove the files in files from current archive */
void KarchiveurApp::slotRemoveThoseFiles( QStrList files ){
	led->setColor(QColor("red"));
  slotStatusMsg(i18n("Deleting selected files..."));

	if(operation!=NOARCHIVE)
	{
		QString txt;
		QStrList strlist;

		operation=STANDBY;
		archiveobj->removeFilesFromArchive(files);
		operation=STANDBY;
	}

	displayArchiveContent();
	led->setColor(QColor("green"));
  displayMessageArchiveSize();
}

/** a process in CArchive has ended. So make
the necessary with the operation variable */
void KarchiveurApp::slotEndProcess(){
	int kindofCompressor=archivechoice->getKindOfCompressor();
  kdDebug()<<QString("slotEndProcess: operation= %1").arg(operation)<<endl;

	if(operation==SCRIPT)
	{
		progress->setProgress(0);

		QString nomfichselect;
		QString repbase;
		QString repbasetmp;
		CScript objetscript;
		QListViewItem *li;
		int typescript;

		operation=STANDBY;
		repbase="";
		li=view->currentItem();

		if((li==NULL)||(li->text(0)==".."))
		{
			li=view->firstChild();
			while(li)
			{
				if(strcmp(li->text(0),"..")!=0)
					break;
				li=li->nextSibling();
			}
		}
		nomfichselect=li->text(0);
		if(nomfichselect=="..")
		{
			led->setColor(QColor("orange"));
			return;
		}
		if((kindofCompressor==TAR)||(kindofCompressor==TARGZ)||(kindofCompressor==TARBZ2)||(kindofCompressor==ZIP))
		{
			repbasetmp=li->text(6);
			repbasetmp.remove(0,1);
			nomfichselect=repbasetmp+nomfichselect;
			li=view->firstChild();
			repbase=li->text(6);
			while(li)
			{
				repbasetmp=li->text(6);
				if(repbase.length()>repbasetmp.length())
					repbase=repbasetmp;
				li=li->nextSibling();
			}
			repbase.remove(0,1);
		}

		repbase=reptmp+repbase;
		kdDebug()<<QString("StartingWizard: base path %1").arg(repbase)<<endl;

		typescript=objetscript.demarrerAssistant(nomfichselect,repbase,archivechoice->getArchiveName());
		if(typescript==2)
		{
			//User has choosed to convert the archive
			doArchiveConversion(objetscript.getTypeConversion(),objetscript.getDirectoryConversion());
		}
		else if(typescript==3)
		{
			//User has choosed to split/unsplit archive
			if(objetscript.getDirectoryCutt()!=NULL)
				doCuttUnCutt(objetscript.getDirectoryCutt(),objetscript.getCuttSize());
		}
		led->setColor(QColor("green"));
	}
	else if(operation==CONVERSION)
	{
    //Now the files are extracted to a temp dir, let's start the conversion !
    doArchiveConversion();
    return;
	}
	else if((operation==EXTRACTING)&&(extraction->viewExtractedDirectoryInKonqueror()))
	{
    QString extract_directory;
    QString base_directory;
    extract_directory = extraction->getAccessPath();
    if((archivechoice->getKindOfCompressor()==TAR)||(archivechoice->getKindOfCompressor()==TARGZ)||(archivechoice->getKindOfCompressor()==TARBZ2))
    {
      base_directory=view->firstChild()->text(6);
      base_directory=base_directory.left(base_directory.find('/',1));
      if(base_directory.startsWith(" "))
      	base_directory.remove(0,1);
      extract_directory=extract_directory+"/"+base_directory;
    }
    process->clearArguments();
    *process<<"konqueror";
    *process<<extract_directory;
    process->start(KProcess::DontCare);
  }
  else if(operation==TESTING)
  {
    if(archiveobj->getErrors().isEmpty())
      KMessageBox::information(this,i18n("None errors found !"),i18n("Everything is OK !"));
    else
      KMessageBox::informationList(this,i18n("The following errors occurred:"),archiveobj->getErrors());
  }
  else if(operation==READING)
  {
    if(!archiveobj->getErrors().isEmpty())
      KMessageBox::informationList(this,i18n("Some errors occurred while opening archive:"),archiveobj->getErrors());
  }
  operation=STANDBY;
  kdDebug()<<QString("Leaving slotEndProcess")<<endl;
}

/** Initialize the actions and the GUI */
void KarchiveurApp::initActions(){
  fileNewWindow=new KAction( i18n("New &Window"), kapp->miniIcon(), 0, this, SLOT(slotFileNewWindow()), actionCollection(),"file_new_window");
  fileNewArchive=new KAction( i18n("&New Archive"), "filenew", 0, this, SLOT(slotFileNew()), actionCollection(),"file_new_archive" );
  fileOpenArchive=new KAction( i18n("&Open..."), "fileopen", 0, this, SLOT(slotFileOpen()), actionCollection(),"file_open_archive" );
  fileSearchArchive=new KAction( i18n("Search for archives"), "find", 0, this, SLOT(slotFileSearch()), actionCollection(),"file_search_archive" );
  fileCryptArchive=new KAction( i18n("Crypt current archive"), "encrypted", 0, this, SLOT(slotCryptArchive()), actionCollection(), "file_crypt_archive" );
  fileMailArchive=new KAction( i18n("Mail to..."), "mail_generic", 0, this, SLOT(slotMailArchive()), actionCollection(), "file_mail_archive" );
  fileOpenRecent = KStdAction::openRecent( this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection(), "file_open_recent_kar");
  fileConvertArchive=new KAction( i18n("Convert &as"), "filesave", 0, this, SLOT(slotFileSaveAs()), actionCollection(),"file_convert_archive" );
  fileInfosArchive=new KAction( i18n("View archive infos"), "documentinfo", 0, this, SLOT( slotDisplayArchiveInfos()), actionCollection(),"file_infos_archive" );
  filePrint = KStdAction::print( this, SLOT(slotFilePrint()), actionCollection(), "file_print_kar");
  fileCloseArchive=new KAction( i18n("C&lose Window"), "fileclose", 0, this, SLOT( slotFileClose()), actionCollection(),"file_close_archive" );
  fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());

  ///////////////////////////////////////////////////////////////////
  // menuBar entry archiveMenu
  archiveExtract=new KAction(i18n("Extract To"), "extract", 0, this, SLOT( slotExtractArchive()), actionCollection(),"archive_extract" );
  archiveAdd=new KAction(i18n("Add files"), "compress", 0, this, SLOT( slotAddFiles()), actionCollection(),"archive_add" );
  archiveView=new KAction(i18n("View file"), "exec", 0, this, SLOT( slotDisplayFile()), actionCollection(),"archive_view" );
  archiveViewSelection=new KAction(i18n("View selected"), "view", 0, this, SLOT( slotDisplaySelectedFiles()), actionCollection(),"archive_view_selection" );
  archiveDeleteSelection=new KAction(i18n("Delete Files"), "editcut", 0, this, SLOT( slotRemoveWithUndoSelectedFiles()), actionCollection(),"archive_delete_selection" );
  archiveSetPassword=new KAction(i18n("Set Password"), "encrypted", 0, this, SLOT( slotSetPassword()), actionCollection(),"archive_password" );
  archiveDeleteToTrashSelection=new KAction(i18n("Move to trash"), "edittrash", 0, this, SLOT( slotRemoveToTrashSelectedFiles()), actionCollection(),"archive_delete_to_trash_selection" );
  archiveWizard=new KAction(i18n("Launch the wizard"), "wizard", 0, this, SLOT( slotWizard()), actionCollection(),"archive_wizard" );
  archiveProprieties=new KAction(i18n("Proprieties"), 0, this, SLOT( slotDisplayFileProprieties()), actionCollection(),"archive_proprieties" );
  archiveTest=new KAction(i18n("Test archive"), "documentinfo", 0, this, SLOT( slotTestArchive()), actionCollection(),"archive_test" );

  ///////////////////////////////////////////////////////////////////
  // menuBar entry editeMenu
  editUndo=KStdAction::undo( this, SLOT( slotUndo()), actionCollection(),"edit_undo_kar" );
  editRedo=KStdAction::redo( this, SLOT( slotRedo()), actionCollection(),"edit_redo_kar" );
  editCut=KStdAction::cut( this, SLOT( slotCutt()), actionCollection(),"edit_cut_kar" );
  editCopy=KStdAction::copy( this, SLOT( slotCopy()), actionCollection(),"edit_copy_kar" );
  editPaste=KStdAction::paste( this, SLOT( slotPaste()), actionCollection(),"edit_paste_kar" );
  editSelectAll=KStdAction::selectAll( this, SLOT( slotSelectAll()), actionCollection(),"edit_select_all_kar" );
  editUnselectAll=new KAction(i18n("Select none"), 0, this, SLOT( slotUnselectAll()), actionCollection(),"edit_unselect_all" );
  editFind=new KAction(i18n("Find..."), "find", 0, this, SLOT( slotUseFind()), actionCollection(),"edit_find_kar" );

  ///////////////////////////////////////////////////////////////////
  // menuBar entry diskMenu
	diskSplit=new KAction(i18n("Split/Unsplit"), 0, this, SLOT(slotDiskCutUncut()), actionCollection(), "disk_split");

  ///////////////////////////////////////////////////////////////////
  // menuBar entry viewMenu
  viewToolbar=new KToggleAction(i18n("&Toolbar"), 0, this, SLOT(slotViewToolBar()), actionCollection(), "view_toolbar" );
  viewStatusbar=new KToggleAction(i18n("&Statusbar"), 0, this, SLOT(slotViewStatusBar()), actionCollection(), "view_statusbar" );
  viewArchiveBrowser=new KToggleAction(i18n("&File browser"), 0, this, SLOT(slotViewButineur()), actionCollection(), "view_archive_browser" );
  viewPreferences=new KAction(i18n("preferences"), "configure", 0, this, SLOT(slotSetOptions()), actionCollection(), "view_preferences" );
  viewUpdate=new KAction(i18n("Check for updates"), kapp->miniIcon(), 0, this, SLOT(slotCheckUpdate()), actionCollection(), "view_update" );
  KStdAction::configureToolbars(this, SLOT(slotEditToolbars()), actionCollection(), "view_edit_toolbar" );

  fileNewWindow->setStatusText(i18n("Opens a new application window"));
  fileNewArchive->setStatusText(i18n("Creates a new archive"));
  fileOpenArchive->setStatusText(i18n("Opens an archive"));
  fileSearchArchive->setStatusText(i18n("Search for existing archive"));
  fileConvertArchive->setStatusText(i18n("Convert current archive format to..."));
  archiveDeleteSelection->setStatusText(i18n("Delete selected files"));
  archiveSetPassword->setStatusText(i18n("Set Password for this archive"));
  archiveExtract->setStatusText(i18n("Extract to..."));
  archiveAdd->setStatusText(i18n("Add files to this archive"));
  archiveView->setStatusText(i18n("View the current file"));
  archiveViewSelection->setStatusText(i18n("View all selected files"));
  fileInfosArchive->setStatusText(i18n("Display info on current archive"));
  archiveWizard->setStatusText(i18n("Launch a wizard to configure & compile this archive..."));
  diskSplit->setStatusText(i18n("Split archive in 1.4Mb blocks"));
  viewArchiveBrowser->setStatusText(i18n("Enables / disables the file navigator"));
  viewPreferences->setStatusText(i18n("Modify options of KArchiver"));
  editCut->setStatusText(i18n("move the files that were copied to the clipboard"));
  editCopy->setStatusText(i18n("Extract selected files and copy them to the clipboard"));
  editPaste->setStatusText(i18n("Add the files that were copied to the clipboard"));
  editFind->setStatusText(i18n("Find a filename in this archive"));
  editUndo->setStatusText(i18n("Undo the last operation"));
  editRedo->setStatusText(i18n("Redo the last operation"));

  createGUI();
}

void KarchiveurApp::slotEditToolbars()
{
  KEditToolbar dlg(actionCollection());
  if (dlg.exec())
    createGUI();
}

/** Extract the archives @archives to the directory @extractdir.
	If @promptdir is true, we'll show the extract dialog, to let the user
	choosing its extract dir */
void KarchiveurApp::extractFromCommandLine( QStrList archives, bool promptdir, QCString extractdir, bool chooseExtractDirFromArchive ){
	QStrList strlistWithoutPath;	//Contains the names WITHOUT their path of dropped files
	QStrList strlistWithPath;	//Contains the names WITH their path of dropped files
	QString str;
	QString chemin;
	QString archivename;
	QString message;
	QString nom;
	QString tmp=NULL;
	int kindofCompressor;
  KArchive *archive=NULL;
  CFileInfo fileinfo;
	QStringList strliste=QStringList::fromStrList(archives);

  strlistWithoutPath.clear();
  strlistWithPath.clear();

  for ( QStringList::Iterator it=strliste.begin(); it!=strliste.end(); ++it)
    {
      kdDebug()<<QString("ExtractFromCmdLine:%1*").arg(*it)<<endl;
      str=openRemoteFile(KURL((*it)));
      strlistWithPath.append(str);
      //Let's keep only the filename
      fileinfo.setFile(str);
      str=fileinfo.fileName();
      strlistWithoutPath.append(str);
      chemin=(*it);
    }

  //The path, useful for the QStrList without path
  fileinfo.setFile(chemin);
  chemin=fileinfo.dirPath();
  chemin=chemin.right(chemin.length()-chemin.find(':')-1);
  archivechoice->setFullPath(chemin);
  navigateur->setPath(archivechoice->getAccessPath());
  kdDebug()<<QString("path:%1 *operation*%2 *").arg(chemin).arg(operation)<<endl;

  //We'll check if we have dragged only one file: a valid archive
  archivename=archivechoice->getArchiveName(); //Backup current archive name

  if(strlistWithPath.count()!=1)
  	return;
      //We have dropped one file and it is an archive, so modify the dialog box
      archivechoice->setArchiveName(strlistWithPath.first());
	if(archivechoice->findKindOfCompressor()==1)
	{
		slotStatusMsg(i18n("Unknown archive type"));
		operation=NOARCHIVE;
		led->setColor(QColor("orange"));
		return;
	}

	operation=STANDBY;

	led->setColor(QColor("red"));


	nom=archivechoice->getArchiveName();
	setCaption(nom);

  fileOpenRecent->addURL(KURL(archivechoice->getArchiveName()));
	navigateur->setPath(archivechoice->getCurrentArchiveDirectory());

	view->clear();
	for(int i=0;i<7;i++)
		view->setColumnWidth(i,50);
	operation=READING;

	kindofCompressor=archivechoice->getKindOfCompressor();

	if(archiveobj!=NULL)
		delete archiveobj;
kdDebug()<<QString("ExtrCmdLine:Kind of compressor: %1\n").arg(kindofCompressor);
	switch(kindofCompressor)
	{
		case TARGZ:
			archiveobj=new CTarGz();
			break;
		case TARBZ2:
			archiveobj=new CTarBz2();
			break;
		case TAR:
			archiveobj=new CTar();
			break;
		case ZIP:
			archiveobj=new CZip();
			break;
		case ARJ:
			archiveobj=new CArj();
			break;
		case LHA:
			archiveobj=new CLha();
			break;
		case RAR:
			archiveobj=new CRar();
			break;
		case GZIP:
			archiveobj=new CGz();
			break;
		case BZIP2:
			archiveobj=new CBz2();
			break;
	}
	connect(archiveobj, SIGNAL(archiveReadEnded()), this, SLOT(slotFileQuit()));
	archiveobj->setReadArchiveWithStream(readarchivewithstream);
	archiveobj->setArchiveName(archivechoice->getArchiveName());
//	archiveobj->displayArchiveContent();
  extraction->displayPath();
  operation=STANDBY; //not set to EXTRACTING as we should call  slotEndProcess. However, heren slotFileQuit seems to be called first

   if(promptdir)
   {
  		if(extraction->exec())
  		{
	      archiveobj->extractArchive(extraction->getAccessPath(),extraction->extractAll(),tmp);
  		}
  		else
  			return;
  	}
  	else
  	{
      QString extr(extractdir);

      if(chooseExtractDirFromArchive)
      {
        //Special feature: if the / of the archives doesn't contain a top-dir but many files,
        //let's create a top-directory
        if((kindofCompressor==TAR)||(kindofCompressor==TARGZ)||(kindofCompressor==TARBZ2))
          archive=new KTar(archivechoice->getArchiveName());
        else if(kindofCompressor==ZIP)
          archive=new KZip(archivechoice->getArchiveName());

        if((archive!=NULL)&&(archive->open(IO_ReadOnly)))
        {
          CFileInfo fileInfo(archivechoice->getArchiveName());
          QString archive_base_name=fileInfo.baseName();
          QDir dir(chemin);
          const KArchiveDirectory*  archive_tar_dir=archive->directory();
          const QStringList  archive_tar_entries=archive_tar_dir->entries();
          if(archive_tar_entries.count()>1)
          {
            dir.mkdir(archive_base_name);
            chemin+="/"+archive_base_name+"/";
          }
        }
        extr=chemin;
      }
      archiveobj->extractArchive(extr,EXTRACTANDOVERWRITE,tmp);
    }
}

/** Check wether karchiver is lauched for the first time. If so, launch
a wizard to configure the main features */
void KarchiveurApp::checkFirstInstall(){
  int rep=1;
  QString arkpath, arkpath2, arkpath3;
  CFirstInstallWizard firstinstallwizard(this, "1st_install", true);

  config->setGroup("First Time Install");
  if( config->readBoolEntry("first run", true) )
  	if( firstinstallwizard.exec() )
  {
	  config->setGroup("First Time Install");
	  config->writeEntry("first run", false);

		config->setGroup("OPTIONS");
		if(firstinstallwizard.rb_wind_like->isChecked())
			config->writeEntry("SelectionMode",1);
		else
			config->writeEntry("SelectionMode",0);

		config->setGroup("LASTDIRS");
		if(firstinstallwizard.rb_extract_lastdir->isChecked())
			rep=0;
		else if(firstinstallwizard.rb_extract_homedir->isChecked())
			rep=1;
		else if(firstinstallwizard.rb_extract_shelldir->isChecked())
			rep=2;
		config->writeEntry("OptionExtraction",rep);

		if(firstinstallwizard.rb_open_lastdir->isChecked())
			rep=0;
		else if(firstinstallwizard.rb_open_homedir->isChecked())
			rep=1;
		else if(firstinstallwizard.rb_open_shelldir->isChecked())
			rep=2;
		config->writeEntry("OptionOuverture",1);

		if(firstinstallwizard.cb_use_karchiver->isChecked())
		{
			arkpath=KGlobal::dirs()->findResource("data", "konqueror/servicemenus/arkservicemenu.desktop");
			arkpath3=KGlobal::dirs()->findResourceDir("data", "konqueror/servicemenus/arkservicemenu.desktop");
			arkpath2=KGlobal::dirs()->findResource("services", "arkpart.desktop");
			kdDebug()<<QString("Will gzip %1 %2\n").arg(arkpath).arg(arkpath2);
			KShellProcess proc;

      proc.clearArguments();
      proc << "kdesu";
      proc << "-c";
			proc << "'";
      proc << "mkdir";
      proc << arkpath3+"konqueror/servicemenus/ark";
      proc << ";";
      proc << "mv";
      proc << arkpath;
      proc << arkpath3+"konqueror/servicemenus/ark";
      proc << ";";
			proc << "gzip" << arkpath2;
			proc << "'";
			proc.start(KProcess::Block);

      KMessageBox::information(this, i18n("Has gzipped ark files:\n")+arkpath+"\n"+arkpath2, i18n("KArchiver setup"));
		}
	config->sync();
	readOptions();
	}
}

/** Automaticaly create a compressed file containing @files.
If there is a single file or directory, create an archive
named files.tar.gz. Otherwise, prompt the user for a name */
void KarchiveurApp::createArchiveFromCommandLine( QStrList files, QString preferedtype ){
  QStrList strlistWithoutPath;	//Contains the names WITHOUT their path of dropped files
  QStrList strlistWithPath;	//Contains the names WITH their path of dropped files
  QString str;
  QString chemin;
  CAddFiles dlgTypeAction;
  QString archivename;
  int kindofCompressor;
  QStringList strliste;

  strliste=QStringList::fromStrList(files);
  strlistWithoutPath.clear();
  strlistWithPath.clear();

  for ( QStringList::Iterator it=strliste.begin(); it!=strliste.end(); ++it)
    {
      kdDebug()<<QString("Drop:%1*").arg(*it)<<endl;
      str=openRemoteFile(KURL((*it)));
      strlistWithPath.append(str);
      //Let's keep only the filename
      str=str.right(str.length()-str.findRev('/',str.length()-2)-1);
      if(str.right(1)=="/")
        str=str.remove(str.length()-1,1);
      strlistWithoutPath.append(str);
      chemin=(*it);
    }

  //The path, useful for the QStrList without path
  chemin=chemin.left(chemin.findRev('/',chemin.length()-2));
  chemin=chemin.right(chemin.length()-chemin.find(':')-1);
  archivechoice->setFullPath(chemin);
  navigateur->setPath(archivechoice->getAccessPath());
  kdDebug()<<QString("path:%1 *operation*%2 *").arg(chemin).arg(operation)<<endl;

  //We'll check if we have dragged only one file: a valid archive
  archivename=archivechoice->getArchiveName(); //Backup current archive name
  if(preferedtype.isNull())
    preferedtype=preferedArchiveType;
  if(!preferedtype.startsWith("."))
    preferedtype="."+preferedtype;
    
  if(operation==NOARCHIVE)
    dlgTypeAction.canAddToCurrentArchive(false);

  if(strlistWithPath.count()==1)
  {
     //We have dropped one file; so create archivename from it.
     kdDebug()<<("Will create an archive from this file\n");
     operation=NOARCHIVE;
     archivename=strlistWithPath.first();
     archivename+=preferedtype;
     archivechoice->setArchiveName( archivename );
     archivechoice->findKindOfCompressor();
  }
  else
  {
    if(!dlgTypeAction.exec())
    {
        led->setColor(QColor("orange"));
        operation=STANDBY;
        return;
    }
    switch(dlgTypeAction.getAddFilesChoice())
      {
      case 0:
        //Add to this archive
        archivechoice->setArchiveName(archivename);
        archivechoice->findKindOfCompressor();
        operation=STANDBY;
        break;
      case 1:
        //Add to another archive
        archivechoice->setArchiveName(dlgTypeAction.getOtherArchiveName());
        archivechoice->findKindOfCompressor();
        kdDebug()<<QString("Add to archive:%1").arg(dlgTypeAction.getOtherArchiveName())<<endl;
        if(archiveobj!=NULL)
  			delete archiveobj;

       switch(archivechoice->getKindOfCompressor())
      	{
         	case TARGZ:
         	  archiveobj=new CTarGz();
         	  break;
         	case TARBZ2:
         	  archiveobj=new CTarBz2();
         	  break;
         	case TAR:
         	  archiveobj=new CTar();
         	  break;
         	case ZIP:
         	  archiveobj=new CZip();
         	  break;
         	case ARJ:
         	  archiveobj=new CArj();
         	  break;
         	case LHA:
         	  archiveobj=new CLha();
         	  break;
         	case RAR:
         	  archiveobj=new CRar();
         	  break;
         	case GZIP:
         	  archiveobj=new CGz();
         	  break;
         	case BZIP2:
         	  archiveobj=new CBz2();
         	  break;
         	default:
         	  led->setColor(QColor("orange"));
         	  operation=NOARCHIVE;
         	  return;
         	  break;
      	}
        connect(archiveobj, SIGNAL(archiveReadEnded()), this, SLOT(slotEndProcess()));
        archiveobj->setReadArchiveWithStream(readarchivewithstream);
        archiveobj->setArchiveName(archivechoice->getArchiveName());
        operation=STANDBY;
        break;
      case 2:
        //Create a new archive
        archivechoice->setArchiveName(dlgTypeAction.getNewArchiveName());
        archivechoice->findKindOfCompressor();
        kdDebug()<<QString("Add to this new archive:%1").arg(dlgTypeAction.getNewArchiveName())<<endl;
        operation=NOARCHIVE;
        break;
      case 3:
        //Open current archive
        operation=STANDBY;
        displayArchiveContent();
        return;
        break;
      case -1:
        led->setColor(QColor("orange"));
        return;
        break;
      }
 	  archivename=archivechoice->getArchiveName();
	}


   //We will create an archive
   led->setColor(QColor("red"));

   if(archivename.isEmpty())
   {
	   slotStatusMsg(i18n("Archive not created"));
     led->setColor(QColor("orange"));
	   return;
   }
   archivechoice->setArchiveName(archivename);
   navigateur->setPath(archivechoice->getAccessPath());
   if(archivechoice->findKindOfCompressor()==1)
   {
	   slotStatusMsg(i18n("Unknown archive type"));
   		led->setColor(QColor("orange"));
	   return;
   }
   kindofCompressor=archivechoice->getKindOfCompressor();
   ///		operation=AJOUTCREATION;
   if(strcmp(archivechoice->getProgrCompresseur(),"tar")==0)
   {
     switch(kindofCompressor)
     {
     case TARGZ:
       archiveobj=new CTarGz();
       break;
     case TARBZ2:
       archiveobj=new CTarBz2();
       break;
     case TAR:
       archiveobj=new CTar();
       break;
     }
     kdDebug()<<QString("Will create Tar based with rel.path:%1*").arg(chemin)<<endl;
     if(includewholepath)
	     archiveobj->createArchive(archivename,strlistWithPath,chemin);
     else
   		  archiveobj->createArchive(archivename,strlistWithoutPath,chemin);
	   operation=ADDCREATE;
   }
   else if(strcmp(archivechoice->getProgrCompresseur(),"unzip")==0)
   {
     archiveobj=new CZip();
     kdDebug()<<QString("Will create Zip based with rel.path:%1*").arg(chemin)<<endl;
     if(includewholepath)
	     archiveobj->createArchive(archivename,strlistWithPath,chemin);
     else
   		  archiveobj->createArchive(archivename,strlistWithoutPath,chemin);
   }
   else
   {
	   KMessageBox::error( this,i18n("Files can be added only in\n .tar, .tar.gz, .tar.bz2"), "karchiveur" );
	   led->setColor(QColor("orange"));
   		return;
   }
   operation=STANDBY;
   //close();
//   delete navigateur;
delete 	archiveobj;
archiveobj=NULL;
exit(0);
   slotFileQuit();
}

/** crypt current archive */
void KarchiveurApp::cryptArchive(){
	CGPG crypt;
kdDebug() << QString("Will crypt %1").arg(archivechoice->getArchiveName()) << endl;
	crypt.setArchiveToProcess( archivechoice->getArchiveName());
	crypt.cryptArchive();
}

/** crypt current archive using gpg */
void KarchiveurApp::slotCryptArchive(){
	cryptArchive();
}

void KarchiveurApp::slotMailArchive() {
	Mail_dialog mail;
	KProcess proc;
	QString adress;
	bool validate;
	int mailoperation;

	proc.clearArguments();

	if(operation==NOARCHIVE)
		return;

	if(mail.exec())
	{
		adress = KLineEditDlg::getText ( i18n("Enter the email address of the recipient"), "someone@example.com", &validate, NULL);
		mailoperation=mail.getKindofFilesToMail();
		if(mailoperation==0)
		{
				//Mail current archive
				proc << "kmail" << "--attach" << KURL::encode_string(archivechoice->getArchiveName()) << adress;
				proc.start(KProcess::DontCare);
		}
		else
		{
  		int i;
  		int k;
  		QListViewItem *li=view->firstChild();
  		QString txt;

  		if((mailoperation==1)&&(view->currentItem()==NULL))
  		{
  		  slotStatusMsg(i18n("First select a file"));
  		  led->setColor(QColor("orange"));
  			return;
  		}
  		
  		proc << "kmail";
			k=view->childCount();

  		for(i=0;i<k;i++)
  		{
  			if((li->text(0)!="..")&&(((mailoperation==1)&&(view->isSelected(li)))||(mailoperation==2)))
  			{
  				txt=li->text(6);
  				txt=txt.remove(0,1);
  				txt+=li->text(0);
  				archiveobj->extractArchive(reptmp, 9, txt);
  				txt=reptmp+txt;
  				proc << "--attach" << KURL::encode_string(txt);
  			}
  			if(i!=k)
  				li=li->nextSibling();
  		}
		  proc << adress;
      proc.start(KProcess::DontCare);
		}
	}
}

/** enable or disable the KAction menus */
void KarchiveurApp::enableMenus(bool b){

  fileCryptArchive->setEnabled(b);
  fileMailArchive->setEnabled(b);
  fileConvertArchive->setEnabled(b);
  fileInfosArchive->setEnabled(b);
  filePrint->setEnabled(b);
  fileCloseArchive->setEnabled(b);

  archiveExtract->setEnabled(b);
  archiveAdd->setEnabled(b);
  archiveView->setEnabled(b);
  archiveSetPassword->setEnabled(b);
  archiveViewSelection->setEnabled(b);
  archiveDeleteSelection->setEnabled(b);
  archiveDeleteToTrashSelection->setEnabled(b);
  archiveWizard->setEnabled(b);
  archiveProprieties->setEnabled(b);
  archiveTest->setEnabled(b);
  
  editUndo->setEnabled(b);
  editRedo->setEnabled(b);
  editCut->setEnabled(b);
  editCopy->setEnabled(b);
  editPaste->setEnabled(b);
  editSelectAll->setEnabled(b);
  editUnselectAll->setEnabled(b);
  editFind->setEnabled(b);
  if(viewbydirectories&&supportDisplayByDirs())
  {
    if(b)
      cbdirectories->show();
    else
      cbdirectories->hide();
  }
}

void KarchiveurApp::useConfigurationWizard( bool use_configuration_wizard )
{
	if(use_configuration_wizard)
 		checkFirstInstall();
}

void KarchiveurApp::slotCheckUpdate()
{
  CUpdate update;
  update.checkUpdate();
}

/** Make all or some files to be extracted */
void KarchiveurApp::slotTestArchive()
{
  QString tmp=NULL;
  if(operation==NOARCHIVE)
  {
  	slotStatusMsg(i18n("First open an archive!"));
  	led->setColor(QColor("orange"));
  	return;
  }

  slotStatusMsg(i18n("Extract files..."));
  led->setColor(KLed::red);

  operation=TESTING;
  archiveobj->extractArchive(reptmp,true,tmp);
}
    
/** User wants to go to a subfolder */
void KarchiveurApp::slotComboDirectoriesActivated(int index){
  QString path;
  int i;

  if(!supportDisplayByDirs())
    return;
    
  for(i=0;i<=index;i++)
    path+=cbdirectories->text(i);
  kdDebug()<<QString("Going to %1, basepath=%2").arg(cbdirectories->text(index)).arg(path)<<endl;
  archiveobj->displayArchiveContent(cbdirectories->text(index),path);
}

/** return true if KIO supports current kind of archive */
bool KarchiveurApp::supportDisplayByDirs(){
  if((archiveobj==NULL)||(cbdirectories==NULL))
    return false;
  
  return archiveobj->supportDisplayByDirs();
  return false;
}

#include "karchiveur.moc"
