/***************************************************************************
                          karchiveurpart.cpp  -  karchiveur component extension
                             -------------------
    begin                : Tue Aug 15 2000
    copyright            : (C) 2001 by Eric Coquelle
    email                : coquelle@caramail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "karchiveurpart.h"
#include "cgpg.h"

#include <klocale.h>
#include <kinstance.h>
#include <kaction.h>
#include <kaboutdata.h>
#include <kparts/partmanager.h>
#include <kdebug.h>
#include <kfileitem.h>


static const char *description = I18N_NOOP("KArchiver for KDE KPart\nAn easy tool to manage all your compressed files!\nReleased under GNU license");

extern "C"
{
    /**
     * This function is the 'main' function of this part.  It takes
     * the form 'void *init_lib<library name>()'.  It always returns a
     * new factory object
     */
    void *init_libkarchiver()
    {
        return new KarchiverFactory;
    }
}

KInstance *KarchiverFactory::s_instance = 0L;

KarchiverFactory::KarchiverFactory()
{
    KGlobal::locale()->insertCatalogue( QString::fromLatin1("karchiver") );
}

KarchiverFactory::~KarchiverFactory()
{
    if (s_instance)
    {
        delete s_instance->aboutData();
        delete s_instance;
    }

    s_instance = 0;
}

QObject *KarchiverFactory::createObject(QObject *parent, const char *name, const char*,
                               const QStringList& )
{
    QObject *obj = new KarchiverPart((QWidget*)parent, name);
    emit objectCreated(obj);
    return obj;
}

/**
 * We need one static instance of the factory for our C 'main'
 * function
 */
 KInstance *KarchiverFactory::instance()
{
    if ( !s_instance )
        s_instance = new KInstance( aboutData() );
    return s_instance;
}

KAboutData *KarchiverFactory::aboutData()
{
  KAboutData *about = new KAboutData("karchiver", "karchiver", VERSION,
				     description, KAboutData::License_GPL,
				     "(c) 2000-2004, Eric Coquelle");

  return about;
}



/* Class KarchiverPart */
KarchiverPart::KarchiverPart(QWidget *parent, const char *name)
    : KParts::ReadOnlyPart(parent, name)
{
    setInstance(KarchiverFactory::instance());

    archivecontent = new  KarchiverPartListView (parent, "main_widget");
    archivecontent->setFocusPolicy(QWidget::ClickFocus);
    setWidget(archivecontent);
    connect(archivecontent, SIGNAL(activatedisplayfiles( bool )), this, SLOT(slotEnableView( bool )) );

    archiveExtract = new KAction(i18n("&Extract"), "extract", 0, archivecontent, SLOT(slotExtractArchive()), actionCollection(), "extract");
    archiveView = new KAction(i18n("&View"), "exec", 0, archivecontent, SLOT(slotDisplayFiles()), actionCollection(), "view");

    m_extension = new KarchiverBrowserExtension(this);
    setXMLFile("karchiver_partui.rc");

    archiveExtract->setEnabled(false);
    archiveView->setEnabled(false);
}

KarchiverPart::~KarchiverPart(){}

bool KarchiverPart::openFile()
{
    archivecontent->openArchive(m_file);
    archiveView->setEnabled(false);
    archiveExtract->setEnabled(true);
    return true;
}

void KarchiverPart::slotEnableView(bool enable)
{
  archiveView->setEnabled( enable );
}


/* ClassKarchiverBrowserExtension*/
KarchiverBrowserExtension::KarchiverBrowserExtension(KarchiverPart *parent)
    : KParts::BrowserExtension(parent, "KarchiverBrowserExtension")
{
}

KarchiverBrowserExtension::~KarchiverBrowserExtension()
{
}


/* ClassKarchiverPartListView*/
KarchiverPartListView::KarchiverPartListView( QWidget *parent, const char *name ) :
	KListView(parent, name)
{
   createFileListView();
	led=new CLed(this,"led");
	progress=new QProgressBar(this);
	progress->setGeometry(width(),0,0,0);

	archiveobj=new CArchive();
	archiveobj->setWidgetListView(this);
	archiveobj->setDisplayIcons(true);
	archiveobj->setKindOfDate(0);
	delete archiveobj;
	archiveobj=NULL;

	fichier=new CArchiveChoice(this);
    setIconText("Karchiver Kparts");
	 QString directory;
	 directory.sprintf("karchivertmp.%d/", getpid());
	 reptmp = locateLocal( "tmp", directory );

	 if (reptmp.isNull())
	   kdWarning() <<i18n("Failed to create temporary "
		    "attachment directory %1")
		    .arg(directory) << endl;

	 directory.sprintf("karchivertmp.%d/extract/", getpid());
	 reptmp = locateLocal( "tmp", directory );

	 if (reptmp.isNull())
	   kdWarning() <<i18n("Failed to create temporary "
		    "attachment directory %1")
		    .arg(directory) << endl;

}

KarchiverPartListView::~KarchiverPartListView()
{
  QString directory;
  KProcess process;
  directory.sprintf("karchivertmp.%d/", getpid());
  reptmp = locateLocal( "tmp", directory );

  process.clearArguments();
  process << "rm" << "-Rf" <<reptmp;
  process.start(KProcess::Block);
  if(archiveobj!=NULL)
    delete archiveobj;
}

/* This slot will extract the archive content */
void KarchiverPartListView::slotExtractArchive()
{
  QString tmp=NULL;

	CExtraction extraction;
  extraction.displayPath();
  if(extraction.exec())
  {
      operation=EXTRACTING;
      archiveobj->extractArchive(extraction.getAccessPath(),extraction.extractAll(),tmp);
  }
  else
		return;
}

/* This slot will extract selected file and view it with its associated software */
void KarchiverPartListView::slotDisplayFiles()
{
		QString txt;
		KFileOpenWithHandler* owh;
		KProcess process;

		QListViewItem *li=currentItem();

		if(li==0)
		  return;

		//Get the path of selected element
		txt=li->text(6);
		if(!txt.isEmpty())
		{
			txt=txt.remove(0,1);
			txt+=li->text(0);
		}
		else
		{
			txt=li->text(0);
			txt=txt.remove(0,1);
		}
		archiveobj->extractArchive(reptmp, 9,txt);
		waitpid(process.getPid(),NULL,WNOHANG);
		if((fichier->getKindOfCompressor()==GZIP)||(fichier->getKindOfCompressor()==BZIP2))
		{
			txt=txt.right(txt.length()-txt.findRev('/')-1);
		}
		txt=reptmp+txt;

		KFileItem fi(0,0,KURL(txt));
		owh=new KFileOpenWithHandler();
		fi.run();
}

/* The selection changed. Check if at least one item is selected, to
	* enable/disable the view action */
void KarchiverPartListView::slotSelectionChanged()
{
  nbselitems=0;
  QListViewItem *it = firstChild();
  while (it)
	{
	  if (it->isSelected())
	      nbselitems++;
	  it = it->itemBelow();
	}
	emit( activatedisplayfiles((nbselitems==0) ? false : true));
}

void KarchiverPartListView::createFileListView()
{
	resize(200,150);
	addColumn(i18n("Name"));
	addColumn(i18n("Size"));
	addColumn(i18n("Time"));
	addColumn(i18n("Date"));
	addColumn(i18n("Owner"));
	addColumn(i18n("Permissions"));
	addColumn(i18n("Path"));
	setMultiSelection(TRUE);
	setColumnWidth(6,100);
	setColumnAlignment(1,Qt::AlignRight);
	setColumnAlignment(2,Qt::AlignRight);
	setColumnAlignment(3,Qt::AlignRight);
	setColumnAlignment(4,Qt::AlignRight);
	setColumnAlignment(5,Qt::AlignRight);
	setShowSortIndicator(true);
	show();

	connect(this, SIGNAL(selectionChanged()), this, SLOT(slotSelectionChanged()));
}

/* Prepare to display archive content */
void KarchiverPartListView::openArchive(const QString & archivename )
{
kdDebug()<<QString("In openArchive:%1\n").arg(archivename);

	fichier->setArchiveName( QString(archivename) );
	displayArchiveContent();
}

void KarchiverPartListView::resizeEvent ( QResizeEvent * newSize) {
	resize( newSize->size() );
}

/** this method will launch the right compressor
to list the file contents
It will then launch the right method to display
the content in the listview */
void KarchiverPartListView::displayArchiveContent()
{
	QString message;
	QString nom;
	int kindofCompressor;

	if(fichier->findKindOfCompressor()==1)
	{
		operation=NOARCHIVE;
		return;
	}
	if(fichier->getKindOfCompressor()==9)
	{
		//This is a crypted archive
		CGPG crypt;
		crypt.setArchiveToProcess( fichier->getArchiveName());
		crypt.decryptArchive();
		fichier->setArchiveName(fichier->getArchiveName().remove(fichier->getArchiveName().length(),4));
  }

	nom=fichier->getArchiveName();

	clear();
	for(int i=0;i<7;i++)
	setColumnWidth(i,50);
	operation=READING;

	kindofCompressor=fichier->getKindOfCompressor();

	if(archiveobj!=NULL)
		delete archiveobj;

	switch(kindofCompressor)
	{
		case TARGZ:
			archiveobj=new CTarGz();
			break;
		case TARBZ2:
			archiveobj=new CTarBz2();
			break;
		case TAR:
			archiveobj=new CTar();
			break;
		case ZIP:
			archiveobj=new CZip();
			break;
		case ARJ:
			archiveobj=new CArj();
			break;
		case LHA:
			archiveobj=new CLha();
			break;
		case RAR:
			archiveobj=new CRar();
			break;
		case GZIP:
			archiveobj=new CGz();
			break;
		case BZIP2:
			archiveobj=new CBz2();
			break;
	}

	archiveobj->setWidgetLed(led);
	archiveobj->setWidgetProgressBar(progress);
	archiveobj->setWidgetListView ( this );
	archiveobj->setOverwriteFilesWhenExtracting(true);
	archiveobj->setIconSize(16);
	archiveobj->setReadArchiveWithStream(false);
	archiveobj->setArchiveName(fichier->getArchiveName());

	connect(archiveobj, SIGNAL(archiveReadEnded()), this, SLOT(slotEndProcess()));

  archiveobj->setReadArchiveWithStream(false);
	archiveobj->setArchiveName(fichier->getArchiveName());
	archiveobj->displayArchiveContent();
}


#include "karchiveurpart.moc"
