/***************************************************************************
                          karchiveurpart.h  -  karchiveur component extension
                             -------------------
    begin                : Tue Aug 15 2000
    copyright            : (C) 2001 by Eric Coquelle
    email                : coquelle@caramail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __KARCHIVEURPART_H__
#define __KARCHIVEURPART_H__

#include <kparts/part.h>
#include <kparts/browserextension.h>
#include <kparts/factory.h>
#include <krun.h>
#include <qprogressbar.h>
#include <qwidget.h>
#include <klistview.h>
#include <kopenwith.h>
#include <kprocess.h>

#include "karchiveur.h"
#include "carchive.h"
#include "cchoixfichier.h"
#include "cled.h"
#include "cextraction.h"

class KarchiverPartListView;

class KarchiverBrowserExtension;

class KarchiverFactory : public KLibFactory
{
    Q_OBJECT
public:
    KarchiverFactory();
    virtual ~KarchiverFactory();

    virtual QObject* createObject(QObject* parent = 0, const char* name = 0, const char* classname = "QObject", const QStringList &args = QStringList());
    static KInstance *instance();
    static KAboutData *aboutData();

private:
    static KInstance *s_instance;
};



class KarchiverPart: public KParts::ReadOnlyPart
{
    Q_OBJECT
public:
    KarchiverPart(QWidget *parent = 0, const char *name = 0);
    virtual ~KarchiverPart();

protected:
    virtual bool openFile();

protected slots:
    void slotEnableView( bool enable );

private:
    KarchiverPartListView *archivecontent;
    KarchiverBrowserExtension *m_extension;

    KAction *archiveExtract;
    KAction *archiveView;
};



class KarchiverBrowserExtension : public KParts::BrowserExtension
{
    Q_OBJECT
    friend class KarchiverPart;
public:
    KarchiverBrowserExtension(KarchiverPart *parent);
    virtual ~KarchiverBrowserExtension();
};



class KarchiverPartListView : public KListView
{
  Q_OBJECT
public:
  KarchiverPartListView( QWidget *parent=0, const char *name=0 );
  virtual ~KarchiverPartListView();
	/* Prepare to display archive content */
  void openArchive(const QString & archivename);

public slots:
	/* This slot will extract the archive content */
  void slotExtractArchive();
	/* This slot will extract selected file and view it with its associated software */
  void slotDisplayFiles();

signals:
  void activatedisplayfiles( bool );

protected:
virtual  void resizeEvent ( QResizeEvent * );

private slots:
	/* The selection changed. Check if at least one item is selected, to
	* enable/disable the view action */
  void slotSelectionChanged();

private:
  void newCaption(const QString& filename);
  void createFileListView();
  void displayArchiveContent();

private:
  bool m_bViewInProgress;

  QString m_strFileToView;
  CArchive *archiveobj;
  CArchiveChoice *fichier;
  int operation;
  CLed *led;
  QProgressBar *progress;
  QString reptmp;
  int nbselitems;
};

#endif
